// 
//  Copyright (C) 1998-2003,2007,2011  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H***************************************************************************
 
  * FILE NAME:   @(#)AsciiRead.hh
  
  * DEVELOPMENT: Chandra X-ray Science Center fef library
                 hhe@cfa.harvard.edu (initial version)
  
  * DESCRIPTION: Header file

 
  * NOTES:
  
        None
  
  * REVISION HISTORY:
  
        Ref. No.        Date
        --------        ----
	2.01            10/22/98


*H****************************************************************************/
#ifndef _ASCIIREAD_HH_
#define _ASCIIREAD_HH_

# include "rmf_datafile.hh"

#include <fstream>
#include <iostream>
#include <iterator>
using namespace std;


extern "C" {
#include "dslib.h"  // define DS_SZ_LINE
}

class AsciiRead : DataFile {

  enum{MAX_COLUMNS = 10, MAX_LINE_BUFFER = DS_SZ_LINE}; 

public:

  AsciiRead(DataType = ASCII);
  ~AsciiRead(){ if(d_table) delete [] d_table;};

  virtual DataFile::ErrorCode open(const char  *filename, Mode = READ);
  virtual void close();
  DataFile::ErrorCode MappingString(char *line,  vector<string>&, 
				    const char *delim=" ");

  short GetNumColumns(){return d_numColumns;};
  void GetColumn( short colNumb,  vector<float> & );
  vector<float> GetRow( short rowNumb, short = 0 );
  vector<float> *GetTable(){ return d_table; };
  vector<string> GetHeader(){ return d_lineHeader;};
  vector<string> GetDataLine(){ ReadDataLine(); return d_sline;};

  string GetOneHeaderLine(char *src){  
    return GetOneHeaderLine( string(src) );  };
  string GetOneHeaderLine(string);
  void SetHeaderMark(const char *line_delim){ d_hdrMark = line_delim;};
  void PrintHeader();
  void PrintData();

  virtual const char *const className() { return ("AsciiRead"); };

protected:

  DataFile::ErrorCode  ReadData(long num_col, short *col_th);
  DataFile::ErrorCode  ReadDataLine();

  short           d_numColumns;
  vector<float>   *d_table;     // 2-D float array: [column][row]
  vector<string>  d_lineHeader; // a line string header
  vector<string>  d_sline    ;  // a line string data
  vector<string>  d_axisName ;  // axis name for a column
  vector<string>  d_axisUnit ;  // axis unit for a column
  string          d_hdrMark;
  ifstream        d_infile;

private:
  DataFile::ErrorCode MappingString(char *line, vector<float>&, const char *delim=" "  );
  DataFile::ErrorCode ReadColumnNameAndUnit(void);

};

#endif

