#ifndef _ARDLIB_H_
#define _ARDLIB_H_	1
/* -*- mode: C; mode: fold -*- */
/*
  This file is part of the Chandra Expmap program suite.
  Copyright (c) 1999-2013,2022 Massachusetts Institute of Technology

  This software was developed by the MIT Center for Space Research
  under contract SV1-61010 from the Smithsonian Institution.

  Permission to use, copy, modify, distribute, and sell this software
  and its documentation for any purpose is hereby granted without fee,
  provided that the above copyright notice appear in all copies and
  that both that copyright notice and this permission notice appear in
  the supporting documentation, and that the name of the Massachusetts
  Institute of Technology not be used in advertising or publicity
  pertaining to distribution of the software without specific, written
  prior permission.  The Massachusetts Institute of Technology makes
  no representations about the suitability of this software for any
  purpose.  It is provided "as is" without express or implied warranty.

  THE MASSACHUSETTS INSTITUTE OF TECHNOLOGY DISCLAIMS ALL WARRANTIES
  WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL THE MASSACHUSETTS
  INSTITUTE OF TECHNOLOGY BE LIABLE FOR ANY SPECIAL, INDIRECT OR
  CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
  OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
  NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifdef __cplusplus
# define CPLUSPLUS_START extern "C" {
# define CPLUSPLUS_STOP }
#else
# define CPLUSPLUS_START
# define CPLUSPLUS_STOP
#endif

CPLUSPLUS_START

#define ARDLIB_VERSION_STRING	"0.8.7-2"
#define ARDLIB_VERSION		807

typedef struct
{
  unsigned short spectrum;
  char spectrum_msg[256];
  unsigned short transform;
  char transform_msg[256];
}
LSF_Parm_Error_Info;

typedef struct
{
   double pos_x;
   double pos_y;
   double theta;
   double phi;
   double sim_x;
   unsigned int row;
   unsigned int off_axis_used;
   char caldb_file[128];
   LSF_Parm_Error_Info err_info;
}
LSF_Parm_Info;

extern int Ardlib_Version;
extern char *ardlib_get_version_string (void);

/*{{{ Initialization Functions */

extern int ardlib_initialize (char *mission, char *obsfile);
extern int ardlib_finalize (void);

/*}}}*/

extern int ardlib_set_parfile (char *file);
extern int ardlib_set_verbosity (int); /* returns old value */

extern int ardlib_write_history (int (*callback)(void *, char *),
				 void *client_data);

/* *value is a malloced pointer */
extern int ardlib_is_option_present (char *options, char *option_name, char **value);
extern char *ardlib_add_option (char *subsys, char *option);

/*{{{ Functions dealing with mirror effective areas */

typedef struct _Ardlib_Mirror_EA_Type Ardlib_Mirror_EA_Type;

extern Ardlib_Mirror_EA_Type *ardlib_open_mirror_ea (char *mirror);
extern int ardlib_close_mirror_ea (Ardlib_Mirror_EA_Type *);

extern int ardlib_compute_mirror_ea (Ardlib_Mirror_EA_Type *,
				     double theta, double phi,
				     float *energies,
				     unsigned int num_energies,
				     float *eff_areas);

/*}}}*/

/*{{{ Detector QE Function */

typedef struct _Ardlib_Det_QE_Type Ardlib_Det_QE_Type;
extern Ardlib_Det_QE_Type *ardlib_open_det_qe (char *);
extern int ardlib_close_det_qe (Ardlib_Det_QE_Type *);

extern int ardlib_compute_det_qe (Ardlib_Det_QE_Type *qet,
				  double x, double y,
				  float *energies, unsigned int num,
				  float *qes);

/*}}}*/

/*{{{ Bad Pixel Functions */

typedef struct _Ardlib_Det_BadPix_Type Ardlib_Det_BadPix_Type;
extern Ardlib_Det_BadPix_Type *ardlib_open_det_badpix (char *);
extern int ardlib_close_det_badpix (Ardlib_Det_BadPix_Type *);
extern int ardlib_det_is_badpix (Ardlib_Det_BadPix_Type *, double, double);

/*}}}*/

/*{{{ Grating Efficiencies */

typedef struct _Ardlib_Grating_Eff_Type Ardlib_Grating_Eff_Type;
extern Ardlib_Grating_Eff_Type *ardlib_open_grating_eff (char *grating, int order);
extern int ardlib_close_grating_eff (Ardlib_Grating_Eff_Type *g);
extern int ardlib_compute_grating_eff (Ardlib_Grating_Eff_Type *g,
				       float *energies, unsigned int num_energies,
				       float *eff);

/*}}}*/

/*{{{ RMFs */

/* Grating RMFs */
typedef struct _Ardlib_Grating_RMF_Type Ardlib_Grating_RMF_Type;
extern int ardlib_compute_grating_rmf (Ardlib_Grating_RMF_Type *, double lambda,
				double *lo, double *hi, unsigned int num,
				double *rmf);

extern Ardlib_Grating_RMF_Type *ardlib_open_grating_rmf (char *grating, int order,
							 int (*width_callback)(double, void *, double *),
							 void *width_callback_client_data,
                                                         LSF_Parm_Info* lsfparm);
extern int ardlib_close_grating_rmf (Ardlib_Grating_RMF_Type *r);

/* Detector RMFs */
typedef struct _Ardlib_RMF_Type Ardlib_RMF_Type;
extern int ardlib_compute_rmf (Ardlib_RMF_Type *, double x, double y,
				double *lo, unsigned int num, double energy,
				double *rmf);

extern Ardlib_RMF_Type *ardlib_open_rmf (char *);
extern int ardlib_close_rmf (Ardlib_RMF_Type *r);

typedef struct _Ardlib_RMF_Gain_Type Ardlib_RMF_Gain_Type;
extern int ardlib_apply_rmf_gain (Ardlib_RMF_Gain_Type *, double chipx, double chipy,
				  double *phas, unsigned int num, double *energies);
extern Ardlib_RMF_Gain_Type *ardlib_open_rmf_gain (char *);
extern int ardlib_close_rmf_gain (Ardlib_RMF_Gain_Type *);

/*}}}*/

/*{{{ Caldb Interface */

typedef struct _Ardlib_Caldb_Type Ardlib_Caldb_Type;
extern int ardlib_caldb_set_obsfile (char *);
extern Ardlib_Caldb_Type *ardlib_caldb_alloc_caldb_type (char *telescope, char *instrument, char *dataproduct);
extern void ardlib_caldb_free_caldb_type (Ardlib_Caldb_Type *);
extern int ardlib_caldb_set_detector (Ardlib_Caldb_Type *, char *fmt, ...);
extern int ardlib_caldb_set_query_expr (Ardlib_Caldb_Type *, char *fmt, ...);
extern char *ardlib_caldb_get_data_file (Ardlib_Caldb_Type *caldb, char *param);

/*}}}*/

CPLUSPLUS_STOP
#endif				       /* _ARDLIB_H_ */
