/*                                                                
**  Copyright (C) 1998-2000,2007  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


/*************************************************************************

 FILENAME   :  ds_acis_config.h
 
 DESCRIPTION:  The file ds_acis_config.h contains defines used by tools
    dealing with acis data. Included in the defines are items such as the
    TLMIN, TLMAX, and TUNITS of the various columns occurring in acis
    data files.   

 NOTES      :  The values used in the TLMIN/TLMAX/TUNITS defines are 
    taken from the ACIS level 1 to archive ICD, rev 1.5 (08/11/98).

*************************************************************************/


#ifndef DS_ACIS_CONFIG_H
#define DS_ACIS_CONFIG_H


/*  The following defines are intended to centralize TLMIN/TLMAX values
 *  for columns in acis data files. They #defines are of the format:
 *  DS_ACIS_<column_name>_<type> where column_name is the name specified 
 *  in the icd (including underscores) and type is either TLMIN or TLMAX.
 *  The defines are listed in the order that the appear in the icd, ie.
 *  although time is a column in many of the data files, it is only
 *  listed under its first occurance in the icd (the events table for
 *  time).  
 *
 *  TLMIN/TLMAX 
 */ 

   /* events (TE/CC common) */
#define DS_ACIS_CCD_ID_TLMIN            0  /* min value for ccd id       */
#define DS_ACIS_CCD_ID_TLMAX            9  /* min value for ccd id       */
#define DS_ACIS_EXPNO_TLMIN             0  /* min value for expno        */
#define DS_ACIS_EXPNO_TLMAX    2147483647  /* max expno value (2^31 -1)  */
#define DS_ACIS_PHAS_TLMIN          -4096  /* min value for phas         */
#define DS_ACIS_PHAS_TLMAX           4095  /* max value for phas         */
#define DS_ACIS_CORN_PHA_TLMIN      -4096  /* min value for corner pha   */
#define DS_ACIS_CORN_PHA_TLMAX       4095  /* max value for corner pha   */
#define DS_ACIS_NODE_ID_TLMIN           0  /* min value for node id      */
#define DS_ACIS_NODE_ID_TLMAX           3  /* max value for node id      */
#define DS_ACIS_DETX_TLMIN            0.5
#define DS_ACIS_DETX_TLMAX         8192.5
#define DS_ACIS_DETY_TLMIN            0.5
#define DS_ACIS_DETY_TLMAX         8192.5
#define DS_ACIS_X_TLMIN               0.5
#define DS_ACIS_X_TLMAX            8192.5
#define DS_ACIS_Y_TLMIN               0.5
#define DS_ACIS_Y_TLMAX            8192.5



   /* events TE mode values */
#define DS_ACIS_TE_CHIPX_TLMIN          1  /* min value for TE chipx     */
#define DS_ACIS_TE_CHIPX_TLMAX       1024  /* max value for TE chipx     */
#define DS_ACIS_TE_CHIPY_TLMIN          1  /* min value for TE chipy     */
#define DS_ACIS_TE_CHIPY_TLMAX       1024  /* max value for TE chipy     */
#define DS_ACIS_TE_TDETX_TLMIN          1  /* min value for TE tdetx     */
#define DS_ACIS_TE_TDETX_TLMAX       8192  /* max value for TE tdetx     */
#define DS_ACIS_TE_TDETY_TLMIN          1  /* min value for TE tdety     */
#define DS_ACIS_TE_TDETY_TLMAX       8192  /* max value for TE tdety     */
#define DS_ACIS_TE_PHA_TLMIN            0  /* min value for TE pha       */
#define DS_ACIS_TE_PHA_TLMAX        36855  /* max value for TE pha       */
#define DS_ACIS_TE_ENERGY_TLMIN         0  /* min value for TE energy    */
#define DS_ACIS_TE_ENERGY_TLMAX   1000000  /* max value for TE energy    */
#define DS_ACIS_TE_FLTGRADE_TLMIN       0  /* min value for TE fltgrade  */
#define DS_ACIS_TE_FLTGRADE_TLMAX     255  /* max value for TE fltgrade  */
   
   /* events CC mode values */
#define DS_ACIS_CC_CHIPX_TLMIN          1  /* min value for CC chipx     */
#define DS_ACIS_CC_CHIPX_TLMAX       1024  /* max value for CC chipx     */
#define DS_ACIS_CC_CHIPY_TLMIN          1  /* min value for CC chipy     */
#define DS_ACIS_CC_CHIPY_TLMAX        512  /* max value for CC chipy     */
#define DS_ACIS_CC_TDETX_TLMIN          1  /* min value for CC tdetx     */
#define DS_ACIS_CC_TDETX_TLMAX       8192  /* max value for CC tdetx     */
#define DS_ACIS_CC_TDETY_TLMIN          1  /* min value for CC tdety     */
#define DS_ACIS_CC_TDETY_TLMAX       8192  /* max value for CC tdety     */
#define DS_ACIS_CC_PHA_TLMIN            0  /* min value for CC pha       */
#define DS_ACIS_CC_PHA_TLMAX        12284  /* max value for CC pha       */
#define DS_ACIS_CC_ENERGY_TLMIN         0  /* min value for CC energy    */
#define DS_ACIS_CC_ENERGY_TLMAX    100000  /* max value for CC energy    */
#define DS_ACIS_CC_FLTGRADE_TLMIN       0  /* min value for CC fltgrade  */
#define DS_ACIS_CC_FLTGRADE_TLMAX       3  /* max value for CC fltgrade  */

   /* exposure stats */
#define DS_ACIS_EXPTIME_TLMIN       0.001  /* min exptime value (1^-3)   */
#define DS_ACIS_EXPTIME_TLMAX        10.0  /* max exptime value          */
#define DS_ACIS_EVTSENT_TLMIN           0  /* min value for evtsent      */
#define DS_ACIS_EVTSENT_TLMAX  2147483647  /* max evtsent value (2^31 -1)*/
#define DS_ACIS_THR_PIX_TLMIN           0  /* min value for thr_pix      */
#define DS_ACIS_THR_PIX_TLMAX     1048576  /* max thr_pix value (2^20)   */
#define DS_ACIS_DROP_AMP_TLMIN          0  /* min value for drop_amp     */
#define DS_ACIS_DROP_AMP_TLMAX 2147483647  /* max drop_amp val (2^31 -1) */
#define DS_ACIS_DROP_POS_TLMIN          0  /* min value for drop_pos     */
#define DS_ACIS_DROP_POS_TLMAX 2147483647  /* max drop_pos val (2^31 -1) */
#define DS_ACIS_DROP_GRD_TLMIN          0  /* min value for drop_grd     */
#define DS_ACIS_DROP_GRD_TLMAX 2147483647  /* max drop_grd val (2^31 -1) */
#define DS_ACIS_BERR_SUM_TLMIN          0  /* min value for berr_sum     */
#define DS_ACIS_BERR_SUM_TLMAX 2147483647  /* max berr_sum val (2^31 -1) */
#define DS_ACIS_OVRCLOCK_TLMIN      -4096  /* min value for berr_sum     */
#define DS_ACIS_OVRCLOCK_TLMAX       4095  /* max berr_sum val (2^31 -1) */
   
   /* dropped exposure */
#define DS_ACIS_MISEVT_TLMIN            0  /* min value for missing event*/
#define DS_ACIS_MISEVT_TLMAX            4  /* max value for missing event*/
#define DS_ACIS_MISEXP_TLMIN            0  /* min value for missing exp  */
#define DS_ACIS_MISEXP_TLMAX            1  /* max value for missing exp  */

   /* mask file */
#define DS_ACIS_COMPONENT_TLMIN         0  /* min value for component    */
#define DS_ACIS_COMPONENT_TLMAX        36  /* max value for component    */
#define DS_ACIS_CHIPX_MASK_TLMIN        1  /* min value for mask chip x  */
#define DS_ACIS_CHIPX_MASK_TLMAX     1024  /* max value for mask chip x  */
#define DS_ACIS_CHIPY_MASK_TLMIN        1  /* min value for mask chip y  */
#define DS_ACIS_CHIPY_MASK_TLMAX     1024  /* max value for mask chip y  */
#define DS_ACIS_WINDOW_TLMIN            0  /* min value for window       */
#define DS_ACIS_WINDOW_TLMAX            5  /* max value for window       */
#define DS_ACIS_SAMP_CYC_TLMIN          0  /* min value for samp_cyc     */
#define DS_ACIS_SAMP_CYC_TLMAX        255  /* max value for samp_cyc     */
#define DS_ACIS_PHAMIN_TLMIN            0  /* min value for phamin       */
#define DS_ACIS_PHAMIN_TLMAX         4095  /* max value for phamin       */
#define DS_ACIS_PHAMAX_TLMIN            0  /* min value for phamax       */
#define DS_ACIS_PHAMAX_TLMAX        65535  /* max value for phamax       */

   /* summary files */ 
#define DS_ACIS_FEPID_TLMIN             0  /* min value for fep_id       */
#define DS_ACIS_FEPID_TLMAX             5  /* max value for fep_id       */
#define DS_ACIS_VIDRESP_TLMIN           0  /* min value for vidresp      */
#define DS_ACIS_VIDRESP_TLMAX           1  /* max value for vidresp      */
#define DS_ACIS_EVT_THR_TLMIN           0  /* min value for evt_thr      */
#define DS_ACIS_EVT_THR_TLMAX        4095  /* max value for evt_thr      */
#define DS_ACIS_SPL_THR_TLMIN           0  /* min value for spl_thr      */
#define DS_ACIS_SPL_THR_TLMAX        4095  /* max value for spl_thr      */
#define DS_ACIS_VID_OFF_TLMIN           0  /* min value for vid_off      */
#define DS_ACIS_VID_OFF_TLMAX        4095  /* max value for vid_off      */
#define DS_ACIS_CCD_ERRS_TLMIN          0  /* min value for ccd_errs     */
#define DS_ACIS_CCD_ERRS_TLMAX          1  /* max value for ccd_errs     */
#define DS_ACIS_FEP_ERRS_TLMIN          0  /* min value for fep_errs     */
#define DS_ACIS_FEP_ERRS_TLMAX        255  /* max value for fep_errs     */




/*  The following defines are intended to centralize the values of the 
 *  tunit column field values for acis data files. 
 *
 *  TUNITS - EVENT/GTI/DEADTIME FACTOR EXTENSIONS 
 */

#define DS_ACIS_TIME_TUNITS          "s"  /* for time column             */
#define DS_ACIS_CHIP_TUNITS      "pixel"  /* units for chip x/y columns  */
#define DS_ACIS_TDET_TUNITS      "pixel"  /* units for tdet x/y columns  */
#define DS_ACIS_DET_TUNITS       "pixel"  /* units for det x/y columns   */
#define DS_ACIS_SKY_TUNITS       "pixel"  /* units for sky x/y columns   */
#define DS_ACIS_PHAS_TUNITS       "adu"   /* units for phas column       */
#define DS_ACIS_PHA_TUNITS        "adu"   /* units for pha column        */
#define DS_ACIS_CORN_PHA_TUNITS   "chan"  /* units for corn_pha column   */
#define DS_ACIS_ENERGY_TUNITS       "eV"  /* units for energy column     */ 
#define DS_ACIS_PI_TUNITS         "chan"  /* units for pi column         */
#define DS_ACIS_STATUS_TUNITS    "coded"  /* units for status column     */

   /* units pertaining to gti extensions */
#define DS_ACIS_START_TUNITS         "s"  /* units for gti start column  */
#define DS_ACIS_STOP_TUNITS          "s"  /* units for gti start column  */

   /* units pertaining to exposure stats file */
#define DS_ACIS_EXPTIME_TUNITS       "s"  /* units for exptime column    */
#define DS_ACIS_OVRCLOCK_TUNITS    "adu"  /* units for ovrclock column   */

   /* units pertaining to mask file */
#define DS_ACIS_PHAMIN_TUNITS     "chan"  /* units for phamin column     */
#define DS_ACIS_PHAMAX_TUNITS     "chan"  /* units for phamax column     */
#define DS_ACIS_OVERLAP_TUNITS   "coded"  /* units for overlap column    */

   /* units pertaining to summary files */
#define DS_ACIS_EVT_THR_TUNITS     "adu"  /* units for evt_thr column    */
#define DS_ACIS_SPL_THR_TUNITS     "adu"  /* units for spl_thr column    */

#endif 
