/*                                                                
**  Copyright (C) 1998-2001,2007  Smithsonian Astrophysical Observatory 
*/                                                                
/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */



/*************************************************************************

 FILENAME   :  ds_hrc_config.h
 
 DESCRIPTION:  The file ds_hrc_config.h contains defines used by tools
    dealing with hrc data. Included in the defines are items such as the
    TLMIN, TLMAX, and TUNITS of the various columns occurring in hrc
    data files.   

 NOTES      :  The values used in the TLMIN/TLMAX/TUNITS defines are 
    taken from the HRC level 1 to archive ICD, rev 1.4 (06/15/98).

*************************************************************************/


#ifndef DS_HRC_CONFIG_H
#define DS_HRC_CONFIG_H


/*  The following defines are intended to centralize TLMIN/TLMAX values
 *  for columns in hrc data files. The defines are listed in the following
 *  manner: any tlmin/tlmax which applies to both hrc-i and hrc-s are listed
 *  first with a prefix of DS_HRC_. Following these are the hrc-i specific
 *  tlmin/tlmax values which are prefixed by DS_HRCI_. Hrc-s specific 
 *  keywords have DS_HRCS_ as a prefix. Certain tlmin/tlmax values (such as
 *  those provided for det and sky coords) are intended as defaults and 
 *  should be replaced by the values based on the configuration of pixlib. 
 *  Note that the defines do not explicitly cast the values to any specific
 *  data type and the user takes responsibility for casting the values to 
 *  the appropriate data types and dealing with potential over/underflow.
 *
 *  TLMIN/TLMAX - EVENT / DEADTIME FACTOR  
 */ 

 /* ranges for events (identical for hrc-i and hrc-s) */ 
#define DS_HRC_AMP_SF_TLMIN        0  /* min value for hrc amp_sf field   */ 
#define DS_HRC_AMP_SF_TLMAX        3  /* max value for hrc amp_sf field   */ 
#define DS_HRC_AV1_TLMIN           0  /* min value for hrc av1 tap        */ 
#define DS_HRC_AV1_TLMAX        4095  /* max value for hrc av1 tap        */ 
#define DS_HRC_AV2_TLMIN           0  /* min value for hrc av2 tap        */ 
#define DS_HRC_AV2_TLMAX        4095  /* max value for hrc av2 tap        */ 
#define DS_HRC_AV3_TLMIN           0  /* min value for hrc av3 tap        */ 
#define DS_HRC_AV3_TLMAX        4095  /* max value for hrc av3 tap        */ 
#define DS_HRC_AU1_TLMIN           0  /* min value for hrc au1 tap        */ 
#define DS_HRC_AU1_TLMAX        4095  /* max value for hrc au1 tap        */ 
#define DS_HRC_AU2_TLMIN           0  /* min value for hrc au2 tap        */ 
#define DS_HRC_AU2_TLMAX        4095  /* max value for hrc au2 tap        */ 
#define DS_HRC_AU3_TLMIN           0  /* min value for hrc au3 tap        */ 
#define DS_HRC_AU3_TLMAX        4095  /* max value for hrc au3 tap        */ 
#define DS_HRC_TDETX_TLMIN         1  /* min value for hrc tdet x pos     */ 
#define DS_HRC_TDETX_TLMAX     16384  /* max value for hrc tdet x pos     */ 
#define DS_HRC_TDETY_TLMIN         1  /* min value for hrc tdet y pos     */ 
#define DS_HRC_TDETY_TLMAX     16384  /* max value for hrc tdet y pos     */ 
#define DS_HRC_PHA_TLMIN           0  /* min value for hrc pha field      */ 
#define DS_HRC_PHA_TLMAX         255  /* max value for hrc pha field      */ 
#define DS_HRC_PI_TLMIN            0  /* min value for hrc pi field       */ 
#define DS_HRC_PI_TLMAX          255  /* max value for hrc pi field       */ 
#define DS_HRC_SUMAMPS_TLMIN       0  /* min value for hrc sumamps field  */ 
#define DS_HRC_SUMAMPS_TLMAX   24570  /* max value for hrc sumamps field  */ 
#define DS_HRC_CHIP_ID_TLMIN       0  /* min value for hrc chip_id field  */ 
#define DS_HRC_CHIP_ID_TLMAX       3  /* max value for hrc chip_id field  */ 

/* range for hrc mask */
#define DS_HRC_COMPONENT_TLMIN     0  /* min value for window     */
#define DS_HRC_COMPONENT_TLMAX     2  /* max value for window     */
#define DS_HRC_WINDOW_TLMIN        0  /* min value for window     */
#define DS_HRC_WINDOW_TLMAX        1  /* max value for window     */
#define DS_HRC_SAMP_CYC_TLMIN      0  /* min value for samp_cyc   */
#define DS_HRC_SAMP_CYC_TLMAX      1  /* max value for samp_cyc   */
#define DS_HRC_PHAMIN_TLMIN        0  /* min value for phamin     */
#define DS_HRC_PHAMIN_TLMAX      255  /* max value for phamin     */
#define DS_HRC_PHAMAX_TLMIN        0  /* min value for phamax     */
#define DS_HRC_PHAMAX_TLMAX      255  /* max value for phamax     */
#define DS_HRC_OVERLAP_TLMIN       0  /* min value for overlap    */
#define DS_HRC_OVERLAP_TLMAX       2  /* max value for overlap    */

 /* ranges specific to hrc-i (events) */
#define DS_HRCI_CRSU_TLMIN         0  /* min value for hrc-i coarse x pos */
#define DS_HRCI_CRSU_TLMAX        63  /* max value for hrc-i coarse x pos */
#define DS_HRCI_CRSV_TLMIN         0  /* min value for hrc-i coarse y pos */
#define DS_HRCI_CRSV_TLMAX        63  /* max value for hrc-i coarse y pos */
#define DS_HRCI_RAWX_TLMIN         0  /* min value for hrc-i raw x pos    */
#define DS_HRCI_RAWX_TLMAX     16383  /* max value for hrc-i raw x pos    */
#define DS_HRCI_RAWY_TLMIN         0  /* min value for hrc-i raw y pos    */
#define DS_HRCI_RAWY_TLMAX     16383  /* max value for hrc-i raw y pos    */
#define DS_HRCI_CHIPX_TLMIN        1  /* min value for hrc-i chip x pos   */
#define DS_HRCI_CHIPX_TLMAX    16384  /* max value for hrc-i chip x pos   */
#define DS_HRCI_CHIPY_TLMIN        1  /* min value for hrc-i chip y pos   */
#define DS_HRCI_CHIPY_TLMAX    16384  /* max value for hrc-i chip y pos   */
#define DS_HRCI_DETX_TLMIN       0.5  /* min value for hrc-i default det x*/
#define DS_HRCI_DETX_TLMAX   32768.5  /* min value for hrc-i default det x*/
#define DS_HRCI_DETY_TLMIN       0.5  /* min value for hrc-i default det y*/
#define DS_HRCI_DETY_TLMAX   32768.5  /* min value for hrc-i default det y*/
#define DS_HRCI_X_TLMIN          0.5  /* min value for hrc-i default sky x*/
#define DS_HRCI_X_TLMAX      32768.5  /* min value for hrc-i default sky x*/
#define DS_HRCI_Y_TLMIN          0.5  /* min value for hrc-i default sky y*/
#define DS_HRCI_Y_TLMAX      32768.5  /* min value for hrc-i default sky y*/
#define DS_HRCI_CHIP_ID_TLMIN      0  /* min value for hrc-i chip_id      */ 
#define DS_HRCI_CHIP_ID_TLMAX      0  /* max value for hrc-i chip_id      */ 

 /* ranges specific to hrc-s (events) */
#define DS_HRCS_CRSU_TLMIN         0  /* min value for hrc-s coarse x pos */
#define DS_HRCS_CRSU_TLMAX        15  /* max value for hrc-s coarse x pos */
#define DS_HRCS_CRSV_TLMIN         0  /* min value for hrc-s coarse y pos */
#define DS_HRCS_CRSV_TLMAX       191  /* max value for hrc-s coarse y pos */
#define DS_HRCS_RAWX_TLMIN         0  /* min value for hrc-s raw x pos    */
#define DS_HRCS_RAWX_TLMAX      4095  /* max value for hrc-s raw x pos    */
#define DS_HRCS_RAWY_TLMIN         0  /* min value for hrc-s raw y pos    */
#define DS_HRCS_RAWY_TLMAX     49151  /* max value for hrc-s raw y pos    */
#define DS_HRCS_CHIPX_TLMIN        1  /* min value for hrc-s chip x pos   */
#define DS_HRCS_CHIPX_TLMAX     4096  /* max value for hrc-s chip x pos   */
#define DS_HRCS_CHIPY_TLMIN        1  /* min value for hrc-s chip y pos   */
#define DS_HRCS_CHIPY_TLMAX    16456  /* max value for hrc-s chip y pos   */
#define DS_HRCS_DETX_TLMIN       0.5  /* min value for hrc-s default det x*/
#define DS_HRCS_DETX_TLMAX   65536.5  /* min value for hrc-s default det x*/
#define DS_HRCS_DETY_TLMIN       0.5  /* min value for hrc-s default det y*/
#define DS_HRCS_DETY_TLMAX   65536.5  /* min value for hrc-s default det y*/
#define DS_HRCS_X_TLMIN          0.5  /* min value for hrc-s default sky x*/
#define DS_HRCS_X_TLMAX      65536.5  /* min value for hrc-s default sky x*/
#define DS_HRCS_Y_TLMIN          0.5  /* min value for hrc-s default sky y*/
#define DS_HRCS_Y_TLMAX      65536.5  /* min value for hrc-s default sky y*/
#define DS_HRCS_CHIP_ID_TLMIN      1  /* min value for hrc-s chip_id      */ 
#define DS_HRCS_CHIP_ID_TLMAX      3  /* max value for hrc-s chip_id      */ 
#define DS_HRCS_TDETX_TLMAX    49368  /* max value for hrc-s tdet x pos   */ 
#define DS_HRCS_TDETY_TLMAX     4096  /* max value for hrc-s tdet y pos   */ 

 /* ranges specific to dead time factor */
#define DS_HRC_DTF_TLMIN         0.0  /* dead time factor dtf column min  */
#define DS_HRC_DTF_TLMAX         1.0  /* dead time factor dtf column max  */



/*  The following defines are intended to centralize the values of the 
 *  tunit column field values for hrc data files. 
 *
 *  TUNITS - EVENT/GTI/DEADTIME FACTOR EXTENSIONS 
 */

/* units pertaining to events extensions */
#define DS_HRC_TIME_TUNITS       "s"  /* for event/deadtime factor time  */
#define DS_HRC_RAW_TUNITS    "pixel"  /* units for event raw x/y columns */
#define DS_HRC_CHIP_TUNITS   "pixel"  /* units for event chip x/y columns*/
#define DS_HRC_TDET_TUNITS   "pixel"  /* units for event tdet x/y columns*/
#define DS_HRC_DET_TUNITS    "pixel"  /* units for event det x/y columns */
#define DS_HRC_SKY_TUNITS    "pixel"  /* units for event sky x/y columns */
#define DS_HRC_PHA_TUNITS     "chan"  /* units for event pha column      */
#define DS_HRC_PI_TUNITS      "chan"  /* units for event pi column       */

/* units pertaining to gti extensions */
#define DS_HRC_START_TUNITS      "s"  /* units for gti start column */
#define DS_HRC_STOP_TUNITS       "s"  /* units for gti start column */

 /* units for deadtime factors HDU */
#define DS_HRC_PROC_EVT_COUNT_UNITS  "count" /* units for deadtime factor */ 
#define DS_HRC_TOTAL_EVT_COUNT_UNITS "count" /* units for deadtime factor */ 
#define DS_HRC_VALID_EVT_COUNT_UNITS "count" /* units for deadtime factor */ 

/* units for hrc mask */
#define DS_HRC_PHAMIN_TUNITS     "chan"   /* units for phamin column  */
#define DS_HRC_PHAMAX_TUNITS     "chan"   /* units for phamax column  */

#endif 
