/*                                                                
**  Copyright (C) 2021  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

#ifndef DS_MEAN_PNT_H
#define DS_MEAN_PNT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <math.h>
#include <float.h>

/* MeanPointing Structure */
typedef struct mean_pointing
{
  double zeropoint[3];     /* reference point */
  double sum_ra_offset;    /* sum of offsets from zero point - RA   */
  double sum_dec_offset;   /* sum of offsets from zero point - DEC  */
  double sum_roll_offset;  /* sum of offsets from zero point - ROLL */
  long   num_points;       /* Number of data points in sum          */
} MeanPointing;

#define ZERO_INIT -999.0

/* ERROR Codes */
#define DS_MPNT_OK 0
#define DS_MPNT_RA_RANGE   700
#define DS_MPNT_DEC_RANGE  701
#define DS_MPNT_ROLL_RANGE 702
#define DS_MPNT_ZERO_SET   703
#define DS_MPNT_NULL_INPUT 704
#define DS_MPNT_BAD_INPUT  705
#define DS_MPNT_NO_DATA    706
#define DS_MPNT_NOT_IMPLEMENTED 1

/* method prototypes */
void ds_pnt_init( MeanPointing* data );
int  ds_pnt_add_data( MeanPointing* data, double* ra, double* dec, double* roll, long npnts );
int  ds_pnt_set_zero( MeanPointing* data, double ra, double dec, double roll );
int  ds_pnt_calc_mean( MeanPointing* data, double* avg_ra, double* avg_dec, double* avg_roll );
int  ds_pnt_zeropoint_isSet( MeanPointing* data );


#ifdef __cplusplus
}
#endif

#endif /* DS_MEAN_PNT_H */
