// 
//  Copyright (C) 1998-2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H****************************************************************************
 
* FILE NAME:    @(#)integral_rmf.hh
 
* DEVELOPEMENT: Chandra Science Center rmf library
                hhe@cfa.harvard.edu (Initial version)

 
* DESCRIPTION:  Header file 

 
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             02/10/98
        0.2             07/10/98
        0.3             08/26/98

*H****************************************************************************/

#ifndef __RMF_INTEGRAL_HH__
#define __RMF_INTEGRAL_HH__

#include "rmf_grids.hh"
#include "eff_grids.hh"
#include "imaging.hh"

extern "C" {
#include "bool.h"
}


//**************************************
//
// class container of Integral-RMF
//
//**************************************
class IntegRMF : public RMFGeneral
{

  typedef struct cell_param_at_axis_s {
    short axis;           // axis ID
    long  numCellsAtAxis; // number of Cells along the axis
    long *cellSize;       // number of grid points b/w two Cells along the axis
    long length;          // number of grid value elements along the axis
    float *grids;         // grid values along the axis
  } cparam_at_axis;

  enum{XFLAG =0, YFLAG=1};


public:
  IntegRMF();
  IntegRMF(dsErrList*, FILE*, short);
  virtual ~IntegRMF();

  dsErrCode calcRMFs();
  void updateIntRMFAt(long row, long col, float val){ d_intrmf[row][col] = val;};
  void updateIntRMFAt(long row, vector<float> &val){
    if( row>d_cparam[YFLAG].numCellsAtAxis)
      return;
    d_intrmf[row].clear(); 
    d_intrmf[row] = val;
  };
  void cleanRMFAtRow(long row){d_intrmf[row].clear();};
  void allocRMFAtRow(long row, long sz){ d_intrmf[row]=vector<float>(sz, 0.0);};

  vector<float> *getIntRMF(long *rowLen){
    *rowLen=d_cparam[YFLAG].numCellsAtAxis;
    return d_intrmf;  };
  vector<float> *getIntRMF(){ return d_intrmf;};
  void setEffGrids(EffGrids *egrids) { d_egrids = egrids; };
  void setFEFObj(FitsEmbeddedFunction *fgrid) { d_fgrid =  fgrid;};
  void printCellParamAtAxis(short axis);


private:
  bool  isZeroArray(long num_rows, vector<float> *array);
  dsErrCode evalRMFValueAlongCellRow(short cell_row);
  dsErrCode integRMFAlongCellRow(short cell_row);
  dsErrCode setConstParams();
  dsErrCode setCellAlongAxis(short xy_flag, short axis);
  void setZero(long sz, float *);

  virtual const char *const className() { return ("IntegRMF"); };

  short d_gridded_axis;        // values are griddled at the axis.
  long  d_rowoffset;
  short d_dimension;

  Imaging *d_image;             //pointer to Imaging object
  float  *d_rmfvalaux;          // auxiliar rmf value array for calc. speed if multi-regions applied
  vector<float> *d_rmfvalue;    // rmf value array in grid row by column
                                // for each Cell Row
  vector<float> *d_intrmf;      // integrated rmfs value in size-1 of user grids
  EffGrids *d_egrids;           // pointer to  Effective-Grids object
  FitsEmbeddedFunction *d_fgrid;// pointer to  FeF object
  cparam_at_axis *d_cparam;
  FILE *d_dbgPtr;

};
#endif //__RMF_INTEGRAL_HH__







