// 
//  Copyright (C) 1998-1999,2001,2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H***************************************************************************
 
  * FILE NAME: DataFile.hh
  
  * DEVELOPMENT: Tools
  
  * DESCRIPTION:
  
  This file contains the definition for
 
  * NOTES:
  
        None
  
  * REVISION HISTORY:
  
        Ref. No.        Date
        --------        ----
	2.01            10/22/98

*H****************************************************************************/
#ifndef _DATAFILE_HH_
#define _DATAFILE_HH_

#include "rmf_string.hh"

class DataFile  {

public:

  enum Mode {READ,WRITE,APPEND};
  enum DataType {UNKNOWN,FITS, ASCII};
  enum ErrorCode{GOOD, OPEN_FAIL, BAD_FILE, NO_DATA, READ_FAIL, WRITE_FAIL,
		 BAD_EXTENSION, BAD_HDU, BAD_INFILE};

 
  DataFile(DataType type){ d_dataType = type;};   
  virtual ~DataFile(){;}; 
 
  // return data type
  DataType getType() { return d_dataType; };
 
  // return current filename
  const char *getFilename(){ return d_fileName.c_str(); };

  // open a data file
  virtual ErrorCode open(const char *filename=NULL, Mode mode=READ) = 0;

  // close a data file
  virtual void close() = 0;


  virtual const char *const className() { return ("DataFile"); };

protected:

  DataType  d_dataType;        // type of data file (enum DataStyle)
  string    d_fileName;        // filename of current working file
};


#endif //_DATAFILE_HH_

