// 
//  Copyright (C) 1998-2001,2003,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


/*H****************************************************************************
 
* FILE NAME:    @(#)rmf_keywords.hh
 
* DEVELOPEMENT: Chandra X-ray  Science Center fef library
                hhe@cfa.harvard.edu (Initial version)

 
* DESCRIPTION:  Header file
  
* NOTES:
 
        <None>
 
* REVISION HISTORY:
 
        Ref. No.        Date
        --------        ----
        0.1             10/97
 
*H****************************************************************************/

#ifndef _RMF_KEYWORDS_HH_
#define _RMF_KEYWORDS_HH_

//local headers (ASCDS)
#include "rmf_string.hh"
#include "rmf_general.hh"


class RMFKeywords: public RMFGeneral
{
  enum KEYWORDS_ORDER{KNAME=0, EXTNSN, DTYPE, VALUE, COMMNT, NUM_ORDER};

public:
  
  RMFKeywords():RMFGeneral(){ SetHeaderDefaults();};
  RMFKeywords(dsErrList* _errLst, FILE* _log, short _verb=1):RMFGeneral(_errLst, _log,_verb){ SetHeaderDefaults();};
  ~RMFKeywords(){};


  RMFGeneral::ERROR ChangeKeywordName(char *keywd, char *new_keywd);
  RMFGeneral::ERROR ChangeKeywordName(const char *keywd, const char *new_keywd){return  ChangeKeywordName( (char *)keywd, (char *)new_keywd);};

  RMFGeneral::ERROR ChangeComment(char *keywd, char *new_comt);
  RMFGeneral::ERROR ChangeComment(const char *keywd, const char *new_comt){return ChangeComment( (char *)keywd, (char *)new_comt);};

  RMFGeneral::ERROR ChangeExtension(char *keywd, char *new_extn);
  RMFGeneral::ERROR ChangeExtension(const char *keywd, const char *new_extn)
  {return ChangeExtension( (char*)keywd, (char*)new_extn);};

  RMFGeneral::ERROR SetKeywordValue_f(char *keywd, float val);
  RMFGeneral::ERROR SetKeywordValue_f(const char *keywd, float val){return SetKeywordValue_f((char *)keywd, val);};

  RMFGeneral::ERROR SetKeywordValue_l(char *keywd, long  val);
  RMFGeneral::ERROR SetKeywordValue_l(const char *keywd, long  val){return SetKeywordValue_l((char *)keywd, val);};

  RMFGeneral::ERROR SetKeywordValue_s(char *keywd, char *val);
  RMFGeneral::ERROR SetKeywordValue_s(const char *keywd, char *val){return SetKeywordValue_s( (char *)keywd, val);};

  RMFGeneral::ERROR RemoveKeyword(char *kwd);
  RMFGeneral::ERROR RemoveKeyword(const char *kwd){return RemoveKeyword( (char *)kwd);};

  RMFGeneral::ERROR RemoveKeyword(long _index);
  RMFGeneral::ERROR MoveKeywordAfter(long _ref, long _index);
  RMFGeneral::ERROR InsertKeywordAfter(long _ref,
				       char *keywd, char *val, char* typ,
				       char *com, char *extn);

  RMFGeneral::ERROR GetKeywordArrayIndex(char *keywd, long *index);

  RMFGeneral::ERROR GetKeywordValue_f(const char *keywd, float *val);
  RMFGeneral::ERROR GetKeywordValue_l(const char *keywd, long  *val);
  RMFGeneral::ERROR GetKeywordValue_s(const char *keywd, char  *val);

  void VolidateKeywords(); 

  RMFGeneral::ERROR Normalization(vector<float> v);
  RMFGeneral::ERROR Normalization(long num, vector<float> v);
  RMFGeneral::ERROR Normalization(long num, float *v);
  // RMFGeneral::ERROR PolInt(long, float*, float*, float,float &, float &);
  RMFGeneral::ERROR LinearInt( long n, float*, float*, long, float*, float* );
  short IsZeroArray(long leng, float* array) ;
  RMFGeneral::ERROR Locate(long,float*, float,long &);
  float ds_interpolate_floats(float, float,float, float,float);
  void PrtCurrentHeader();
  void PrtCurrentHeader(char*);

protected:

  vector<string> d_keywd, d_val, d_com, d_extnsn, d_dtype;
  vector<short> d_extindex[NUM_EXTNSON];  // index to record extensions, 

  char *GetDate_ddmmyyyy(char* a_date);
  char *GetUTCDate_yyyymmdd(char* a_date);
  char *GetUTCTime_hhmmss(char* a_date);
  rmfString d_rmfstr;

private:

  const char *const className() { return ("RMF_KEYWORDS"); };

  //  RMFGeneral::ERROR MappingData(vector<string>);
  // RMFGeneral::ERROR ProcTable(const char *fname, string end_mark= "# STAR_OF_FILE");
  void SetHeaderDefaults();

};


#endif // _RMF_KEYWORDS_HH_




