#!/usr/bin/env perl
# 
#  Copyright (C) 2013-2018,2023  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

##########################################################################
# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
else
{
   # being run from build directory.
   use Cwd qw( abs_path );
   use File::Basename qw( dirname );
   $TEXEC_DIR = dirname( abs_path( $0 ) );
}
unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '046';
my $semantic = 'pixmaskfilt';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

##########################################################################
# Initialize test infrastructure info.
my %infra = dt_init( $id, $script_name, \@ARGV );

# Transfer info to local variables.
my $version = %infra{'version'};
my $testin  = %infra{'testin'};
my $testout = %infra{'testout'};
my $testsav = %infra{'testsav'};
my $logfile = %infra{'logfile'};
my $logfname = %infra{'logfname'};
my $printfile = %infra{'dbgfile'};

# select output stream 
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = ""; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
51,
52,
53,
54,
55,
56,
57,
58,
59,
60,
61,
62,
63,
64,
65,
66,
67,
68,
69,
70,
71,
72,
73,
74,
80,  # to cover SL18, for nonzero-mask bounds
81,82,
83,84,
85,  # to cover SL18, for zero-mask bounds
91,
92,
93, # SL-88
101,
102,
103,
105,
106,
108,
109,
111,
112,
113,
114,
115,
116,
117,
118,
120,
121,
122,
123,
124,
125,
126,
127,
128,
129,
130,
131,
132,
133,
134,
135,
136,
137,
138,
139,
150,
151,
152,
153,
154,
155,
156,
157,
158,
159,
160,
161,
162,
163,
164,
165,
201,
202,
203,
204,
205,
206,
207,
208,
209,
210,
211,
212,
213,  # bug SL81
214,  # bug SL81
215,  # bug SL81
216,  # bug SL81
221,  # RFE SL266
222,  # RFE SL266
223,  # RFE SL266
505,
506,
528,
529,
530,
531,
);

my %test_blocks = ( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],    <<= dmimgcalc test
#          "table" => [ 2, 4 ] },  <<= dmdiff test
# "2" => { "list" => [ ] },...
1 => { "fail" => []},
2 => { "fail" => []},
3 => { "fail" => []},
4 => { "fail" => []},
51 => { "table" => [2,8] },
52 => { "table" => [2,8] },
53 => { "table" => [2,8] },
54 => { "table" => [2,8] },
55 => { "table" => [2]},
56 => { "table" => [2,8] },
57 => { "table" => [1,2]},
58 => { "table" => [1,2]},
59 => { "table" => [2,8]},
60 => { "fail" => []},       # expected failure.. compatibility
61 => { "table" => [2,8]},   # pass, no compatibility checking
62 => { "fail" => []},       # expected failure.. compatibility
63 => { "fail" => []},       # expected failure.. vector definition
64 => { "table" => [2,8]},
65 => { "table" => [2,8]},
66 => { "table" => [2,8]},
67 => { "table" => [2,8]},
68 => { "image" => [1],
  	"table" => [1,2] },
69 => { "image" => [1],
  	"table" => [1,2] },
70 => { "image" => [1],
  	"table" => [1,2] },
71 => { "table" => [2,8] },  # pass, no compatibility checking
72 => { "fail" => [] },      # expected failure
73 => { "image" => [1,2],
	"table" => [1,2] },
74 => { "image" => [1,2],
	"table" => [1,2] },
 80 => { "table" => [2]},    # test '[sky=bounds(mask(nonzeormsk))]' syntax, without exisitng mask, also cover SL18
 81 => { "image" => [1]},    # test '[sky=bounds(mask(my.msk))]' syntax, without exisitng mask
 82 => { "image" => [1]},    # test '[sky=bounds(mask(my.msk))]' syntax, with    exisitng mask
 83 => { "image" => [1,2] }, # dmcopy evtmask1.fits[exclude sky=mask(circle)] imgmask_exc.fits
 84 => { "image" => [1,2] }, # dmcopy evtmask1.fits[        sky=mask(circle)] imgmask_inc.fits
 85 => { "image" => [1]},    # test '[sky=bounds(mask(zeromsk))]' syntax, without exisitng mask, also cover SL18
 91 => { "table" => [2,4,5] },
 92 => { "table" => [2,4,5] },
 93 => { "fail" => []},       # expected failure.. compatibility
101 => { "table" => [2,8]},
102 => { "table" => [2,8]},
103 => { "table" => [2,8]},
105 => { "table" => [2]},
106 => { "table" => [2]},
108 => { "image" => [1,2]},
109 => { "image" => [1,2]},
111 => { "table" => [2,8]},
112 => { "table" => [2,8]},
113 => { "fail"  => []   },       # expected failure.
114 => { "table" => [2,8] },
115 => { "table" => [2,8] },
116 => { "table" => [2] },
117 => { "table" => [2,8] },
118 => { "table" => [2,8] },
120 => { "image" => [1,3] },  # mask rebinning in 2 commands, mask and bin separated
121 => { "image" => [1,3] },  # mask rebinning in 1 command: mask and bin once
122 => { "image" => [1,2] },  # mask rebinning
123 => { "image" => [1,2] },  # mask rebinning
124 => { "image" => [1,2] },  # mask rebinning
125 => { "image" => [1,2] },  # mask rebinning
126 => { "image" => [1,3] },  # mask rebinning
127 => { "image" => [1,3] },  # mask rebinning
128 => { "fail"  => []    },  # expected error 
129 => { "fail"  => []    },  # expected error
130 => { "text"  => []    },  # mask rebinning
131 => { "table" => [2,4,5] },  # masks prepared for rebinning
132 => { "image" => [1,3,4] },  # mask rebinning
133 => { "table" => [2,4,5] },  # mask scale checking in rebinning process
134 => { "image" => [1,2,4] },  # table with 2masks run filter mask in rebinning
135 => { "image"  => [1,3,4]    },  # mask rebinning
136 => { "fail"  => []    },  # expected error
137 => { "fail"  => []    },  # expected error
138 => { "fail"  => []    },  # expected error
139 => { "image" => [1,2] },  # dmcopy with rebinning-masks, bug SL-21
141 => { "image" => [1,2] },  # dmcopy with rebinning-masks, bug SL-21
150 => { "table" => [2,8] },
151 => { "fail" => [] },      # expected failure
152 => { "table" => [2,8] },
153 => { "fail" => [] },      # expected failure
154 => { "table" => [2,8] },
155 => { "fail" => [] },      # expected failure
156 => { "table" => [1,2] },
157 => { "fail" => [] },      # expected failure
158 => { "table" => [2,8] },
159 => { "table" => [2,8] },
160 => { "image" => [1,2] },
161 => { "table" => [2,8] },
162 => { "image" => [1,2] },
163 => { "table" => [2,8] },
164 => { "table" => [2,8] },
165 => { "table" => [2,8] },
201 => { "image" => [1,2],
	 "table" => [1,2] },
202 => { "table" => [2],
         "image" => [5] },
203 => { "image" => [1,2] },  
204 => { "image" => [1,2] },  
205 => { "table" => [2],
         "image" => [8] },
206 => { "table" => [2],
         "image" => [5] },
207 => { "table" => [2],
         "table" => [7] },
208 => { "table" => [2],
         "image" => [5] },
209 => { "table" => [2],        # 209-212: run dmmerge, bug SL-9
         "image" => [3] },
210 => { "table" => [2],
         "image" => [3] },
211 => { "table" => [2],
         "image" => [3] },
212 => { "table" => [2],
         "image" => [3] },   # cover bugs SL19 and SL20
213 => { "table" => [2] },   # SL81, combinning for two polygon components
214 => { "table" => [2],
         "image" => [3],     # SL81, combinning for two mask components, MASK
         "image" => [4] },   # SL81, combinning for two mask components, MASK2
215 => { "table" => [2] },   # SL81, combinning for one polygon component
216 => { "table" => [2],   
         "image" => [3] },   # SL81, combinning for one mask component, MASK
221 => { "image" => [1] },   # SL266, empty-mask-block,  1 Mask subspace expression
222 => { "table" => [2] },   # SL266, empty-mask-block,  Mask&Shapes subspace expression
223 => { "image" => [1] },   # SL266, empty-masks-block, Mask&Shapes|Mask2&Shapes subspace expression
505 => { "image" => [1],
         "table" => [7] },
506 => { "image" => [1] ,
         "table" => [7] },
208 => { "table" => [2],
         "image" => [5] },
528 => { "table" => [2,8] },
529 => { "table" => [2,8] },
530 => { "table" => [2,8] },
531 => { "table" => [2,8] },
);

my %test_words = (
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
$version => "ASCDS_VERSION" 
);


# Find Tool Executable in PATH
my $DMMERGE = find_inpath( 'dmmerge', $ENV{'PATH'}, ':' );
my $DMSTAT = find_inpath( 'dmstat', $ENV{'PATH'}, ':' );
unless ( $DMMERGE && $DMSTAT )
{
   print $printfile "ERROR: Unable to find tool : dmmerge, dmstat\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN dmcopy" );
system( "$PUNLEARN dmmerge" );

dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $err_msg = "";
my $rc;
my $time_user_total = 0;
my $time_sys_total = 0;

my %exten = ("FITS" => "fits");
my %links = ("1" => "indt046m001");
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#######################################################################

# Loop over DM kernels to test under.
my $kref = %infra{'kernels'};
my @kernels = @$kref;
my $kernelName;

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    if ( $test_blocks{$testid} == "" ){ next; }

    # Init per-test variables.
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;

    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.fits";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.fits";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";

    # --------------------------------------------------------------------------------
    # set test command
    my $test_string = "";

    my $maskfile = "";
    my $maskOne = "";
    my $maskTwo = "";
    my $regfile = "";
    my $regOne = "";
    my $regTwo = "";
    my $infile1 = "";
    my $infile2 = "";
    my $infile3 = "";
    my $infile4 = "";
    my $infile5 = "";

    # MCD NOTE: First 31 tests all fail now due to coordinate system mismatch (0.5 pixel shift).
    #  Starting new suite at 101, will replace old tests as they are covered by new examples.
    if ( $testid > 500){ print "Skip Test $testid\n"; next; }
 
    # -------------------------------------------------------
    # BAD Input Tests: Illegal syntax and inappropriate files
    # -------------------------------------------------------
    if ( $testid == 1 )
    {
	# BAD Input Test: specified MASK file is not an image.. (empty string)
	# EXPECTED FAILURE:  Mask file not found;
	$maskfile = "";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask(${maskfile})]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 2 )
    {
	# BAD Input Test: specified MASK file is not an image.. (region file)
	# EXPECTED FAILURE:  Mask block is not image.
	$maskfile = "$testin/ds9.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask(${maskfile})]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 3 )
    {
	# BAD Input Test: specified MASK file does not exist.
	# EXPECTED FAILURE:  Mask file not found; <maskfile>
	$maskfile = "$testin/pixmask_dne.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask(${maskfile})]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 4 )
    {
	# BAD Input Test: specified MASK file is a NULL image.
	# EXPECTED FAILURE: Mask image is not 2-D but 0-D
	$maskfile = "$testin/pixmask_null.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask(${maskfile})]\" outfile=$outfile verbose=0";
    }

    # ------------------
    # BASIC MASK Filter:
    # ------------------
    if ( $testid == 51 )
    {
	# Postage stamp pixel mask (20x25), all pixels ON, on sky(x,y) physical coordinates.
	$maskfile = "$testin/pixmask_20x25_ON.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 52 )
    {
	# Postage stamp pixel mask (20x25), all pixels OFF, on sky(x,y) physical coordinates.
        # Should produce empty output table, with MASK block
	$maskfile = "$testin/pixmask_20x25_OFF.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 53 )
    {
	# Postage stamp pixel mask (20x25), all pixels ON, on sky(x,y) physical coordinates. (EXCLUDE)
        # Should produce full event file with empty box at MASK location
	$maskfile = "$testin/pixmask_20x25_ON.fits";
	$test_string = "$DMCOPY infile=\"${infile}[exclude sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }

    if ( $testid == 54 )
    {
	# Postage stamp pixel mask (20x25), all pixels OFF, on sky(x,y) physical coordinates. (EXCLUDE)
        # exclude of all OFF filter is no-net, result == input.
	$maskfile = "$testin/pixmask_20x25_OFF.fits";
	$test_string = "$DMCOPY infile=\"${infile}[exclude sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 55 )
    {
	# Postage stamp pixel mask (40x30) outside previous filter range (to Left)
	#   - should result in EMPTY table file. NULL sky subspace.
        $maskfile = "$testin/pixmask_40x30_overlap_xl1k.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 56 )
    {
	# Postage stamp pixel mask (20x25) partial overlap with previous filter range.
	$maskfile = "$testin/pixmask_20x25_partial.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 57 )
    {
	# Postage stamp pixel mask (20x25) on IMAGE sky(x,y) physical coordinates.
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 58 )
    {
	# Postage stamp pixel mask (20x25) on IMAGE, Filter on LOGICAL coordinates.
	# MCD NOTE: 1 pixel shift in resulting image sky transform -- OK?
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
	$test_string = "$DMCOPY infile=\"${infile}[(#1,#2)=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 59 )
    {
	# Postage stamp pixel mask (20x25) on TABLE, Mask with incompatible projection type.
        #  - sky coordinate in TABLE is not virtual, no compatibility check required..
        #  - eqpos coordinate definitions do not match; but are irrelevant.
        #  NOTE: subspace MASK block should NOT contain world coordinate system (eqpos)... but does.
        $maskfile = "$testin/pixmask_20x25_diastripe_coordsin.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 60 )
    {
	# Postage stamp pixel mask (20x25) on IMAGE, Mask with incompatible projection type.
        #  - should ERROR due to incompatible masks
        $maskfile = "$testin/pixmask_20x25_diastripe_coordsin.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 61 )
    {
	# Postage stamp pixel mask (20x25) on TABLE, Mask with bin=2
        #  - sky coordinate in TABLE is not virtual, no compatibility check required..
        #  NOTE: subspace MASK block should NOT contain world coordinate system (eqpos)... but does.
        $maskfile = "$testin/pixmask_20x25_diastripe_coordscale.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 62 )
    {
	# Postage stamp pixel mask (20x25) on IMAGE, Mask with bin=2
        #  - should ERROR due to incompatible masks
        $maskfile = "$testin/pixmask_20x25_diastripe_coordscale.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 63 )
    {
	# Postage stamp pixel mask (20x25) with sky(a,b) physical axes; eqpos(ra,dec) world coordinate axes.
        # Apply to event list with sky(x,y) vector column.
	# EXPECTED FAILURE:  vector definition mis-match.
	$maskfile = "$testin/pixmask_colsky.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask(${maskfile})]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 64 )
    {
	# Postage stamp pixel mask (20x25) with chip(chipx,chipy) physical coordinates, applied on compatible event list.
	$maskfile = "$testin/pixmask_colchip.fits";
	$test_string = "$DMCOPY infile=\"${infile}[chip=mask(${maskfile})]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 65 )
    {
	# Postage stamp pixel mask (20x25) with det(detx,dety) physical coordinates, applied on compatible event list.
	$maskfile = "$testin/pixmask_coldet.fits";
	$test_string = "$DMCOPY infile=\"${infile}[det=mask(${maskfile})]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 66 )
    {
	# Postage stamp pixel mask (20x25) with pos(x,y) physical coordinates, applied to event list with sky(x,y) using component pair.
        # EXPECTED FAILURE?; if not, what does output subspace mask contain? (currently does contain POS(x,y))
	$maskfile = "$testin/pixmask_pairxy.fits";
	$test_string = "$DMCOPY infile=\"${infile}[(x,y)=mask(${maskfile})]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 67 )
    {
	# Postage stamp pixel mask (20x25) with pi and pha 1D physical coordinate axes, applied to event list with pi and pha columns using component pair.
	$maskfile = "$testin/pixmask_pairpipha.fits";
	$test_string = "$DMCOPY infile=\"${infile}[(pi,pha)=mask(${maskfile})]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 68 )
    {
        # Postage stamp pixel mask (20x25); Image and Mask have UNIT transform from logical to physical
	$maskfile = "$testin/pixmask_20x25_unit.fits";
	$test_string = "$DMCOPY infile=\"${infile}[pos=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 69 )
    {
        # Postage stamp pixel mask (20x25); Image and Mask have UNIT transform from logical to physical
	# Test dmFLOAT type pixel mask.. includes NaN values
        # Result should be same as #68
	$maskfile = "$testin/pixmask_20x25_unit_float.fits";
	$test_string = "$DMCOPY infile=\"${infile}[pos=mask($maskfile)]\" outfile=$outfile verbose=0";
    }

    if ( $testid == 70 )
    {
        # Postage stamp pixel mask (20x25); Image and Mask have UNIT transform from logical to physical
	# Test dmSHORT type pixel mask.
        # Result should be same as #68
	$maskfile = "$testin/pixmask_20x25_unit_short.fits";
	$test_string = "$DMCOPY infile=\"${infile}[pos=mask($maskfile)]\" outfile=$outfile verbose=0";
    }

    if ( $testid == 71 )
    {
	# Table Event with 
        # Filter using another IMAGE file's subspace (which includes MASK)
        # MASK is scale=2 passed for coords for Table event as Test #61
        # using  file-directive syntax, '@' 
        $maskfile = "$testin/pmout_a08.fits";
	$test_string = "$DMCOPY infile=\"${infile}[\@$maskfile]\"  outfile=$outfile";
    }
    if ( $testid == 72 )
    {
	# @mask directive
	#  UNSUPPORTED
        $maskfile = "$testin/pm.lis";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask(\@$maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 73 )
    {
	# IMAGE - Postage stamp pixel mask (20x25); 
	#  Resulting image/mask should be size of mask
	$maskfile = "$testin/pixmask_20x25_ON.fits";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 74 )
    {
	# IMAGE - Postage stamp pixel mask (20x25); with opt=full
	#  Resulting image/mask should be size of original image.
	#  Content of pixels from mask area should be identical, area outside mask s/b NULL
	$maskfile = "$testin/pixmask_20x25_ON.fits";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile opt=full verbose=0";
    }

    if ( $testid == 80 )  # to cover SL18
    {
	#  bounds(mask=(nonzeromask)) without existing mask
	$maskfile = "$testin/SL18_mask.ellipse";
        $test_string = "$DMCOPY infile=\"${infile}[sky=bounds(mask($maskfile))]\" outfile=$outfile opt=full verbose=0";
    }
    if ( $testid == 85 )  # to cover SL18
    {
	#  bounds(mask=(zeromask)) without existing mask
	$maskfile = "$testin/SL18_mask.zeros";
        $test_string = "$DMCOPY infile=\"${infile}[sky=bounds(mask($maskfile))]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 81 )  
    {
	#  bounds(mask=()) without existing mask
	$maskfile = "$testin/pixmask_broad_thresh.fits";
        $test_string = "$DMCOPY infile=\"${infile}[sky=bounds(mask($maskfile))]\" outfile=$outfile opt=full verbose=0";
    }
    if ( $testid == 82 )  # hold, more work needed. @ 3/12/2015
    {
	#  bounds(mask=()) with existing mask
	#  bounds(circle()) with existing mask
	$maskfile = "$testin/pixmask_broad_thresh_sml.fits";
        $test_string = "$DMCOPY infile=\"${infile}[sky=bounds(mask($maskfile))]\" outfile=$outfile opt=full verbose=0";
    }

    if ( $testid == 83 )  
    {
	#  exclude sky=(mask=(mask2)) with existing mask1
	$maskfile = "$testin/circle.fits";
        $test_string = "$DMCOPY infile=\"${infile}[exclude sky=mask($maskfile)]\" outfile=$outfile opt=full verbose=0";
    }

    if ( $testid == 84 )  
    {
	#  include sky=(mask=(mask2)) with existing mask1
	$maskfile = "$testin/circle.fits";
        $test_string = "$DMCOPY infile=\"${infile}[ sky=mask($maskfile)]\" outfile=$outfile opt=full verbose=0";
    }

    if ( $testid == 91 )  # multi-mask descriptors/subspaces
    {
	$maskOne = "$testin/sky.mask";
	$maskTwo = "$testin/expno_vs_pi.mask";
        $test_string = "$DMCOPY infile=\"${infile}[(x,y)=mask($maskOne), (expno,pi)=mask($maskTwo)]\" outfile=$outfile opt=full verbose=0";
    }

    if ( $testid == 92 )  # multi-mask descriptors/subspaces
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 91;
	$infile = "$testsav/${basename}.$in_ext";
	$savfile = "$testsav/${basename}.fits";

        $test_string = "$DMCOPY infile=\"${infile}\" outfile=$outfile opt=full verbose=0";
   }

   if ( $testid == 93 ) # SL-88: filter with bin size mismatch
   {
   # Single, central pixel IMAGE bin=1, Mask with bin=2
   #  - should ERROR due to incompatible bin size
   $maskfile = "$testin/pixmask_50x50_bin2.fits";
   $test_string = "$DMLIST ${infile}\"[(col1,col2)=mask($maskfile)]\" subspace  verbose=0";
   }
  

    # ------------------
    # MASK-Region Combos:
    # ------------------
    if ( $testid == 101 )
    {
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates in conjunction with non-overlapping region (UNION).
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
        $regfile = "$testin/circle_no_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)||sky=region($regfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 102 )
    {
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates in conjunction with overlapping region (UNION).
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
        $regfile = "$testin/circle_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)||sky=region($regfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 103 )
    {
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates in conjunction with overlapping region (INTERSECT).
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
        $regfile = "$testin/circle_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)&&sky=region($regfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 105 )
    {
	# Postage stamp pixel mask (20x25) outside previous filter ranges (to LL), 
        # in conjunction with non-overlapping region (within data ranges) (UNION)
	#   - should result in filter on region only, no MASK in subspace.
        $maskfile = "$testin/pixmask_20x25_diastripe_ll1k.fits";
        $regfile = "$testin/circle_no_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)||sky=region($regfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 106 )
    {
	# Postage stamp pixel mask (20x25) outside previous filter ranges (to UR), 
        # in conjunction with non-overlapping region (within data ranges) (UNION)
	#   - should result in filter on region only, no MASK in subspace.
	#   - should be SAME results as #105
        $maskfile = "$testin/pixmask_20x25_diastripe_ur1k.fits";
        $regfile = "$testin/circle_no_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)||sky=region($regfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 108 )
    {
	# Postage stamp pixel mask (20x25) on IMAGE, Mask and image bin factor=2...
	# Applied in conjunction with region on image, not overlapping Mask. (UNION)
        $maskfile = "$testin/pixmask_20x25_diastripe_coordscale.fits";
        $regfile = "$testin/circle_no_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)||sky=region($regfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 109 )
    {
	# Postage stamp pixel mask (40x30) on IMAGE with existing MASK in subspace.
        #  Mask and image bin factor=2, with nearly complete overlay.
        #  MASK Consolidation occurs.
        $maskfile = "$testin/pixmask_40x30_overlap_coordscale.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 111 )
    {
	# Postage stamp pixel mask (20x25) on TABLE, using sky vector component syntax.
	# In conjunction with overlapping region using sky vector syntax.
        #  - should be equivalent to #102
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
        $regfile = "$testin/circle_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[(x,y)=mask($maskfile)||sky=region($regfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 112 )
    {
	# Postage stamp pixel mask (20x25) on TABLE, using sky vector syntax.
	# In conjunction with overlapping region using sky vector component syntax.
        #  - should be equivalent to #102
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
        $regfile = "$testin/circle_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)||(x,y)=region($regfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 113 )
    {
	# Postage stamp pixel mask (40x30), on IMAGE result of Test 108, 
        #  Apply new overlapping Mask with bin factor==1
        #  - should ERROR due to incompatible masks.
        $maskfile = "$testin/pixmask_40x30_overlap.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 114 )
    {
        # 
	# Apply mask filter followed by apply region filter 
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates, followed by overlapping region (INTERSECT).
        # Result should be equivalent to single command line (#103)
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
        $regfile = "$testin/circle_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)\]\" outfile=${outfile}_tmp;";
	$test_string .= "$DMCOPY infile=\"${outfile}_tmp\[sky=region($regfile)\]\" outfile=$outfile";
    }
    if ( $testid == 115 )
    {
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates in conjunction with overlapping region (UNION).
        # Order of expression reversed - Results should be same as #102
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
        $regfile = "$testin/circle_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=region($regfile)||sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 116 )
    {
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates in conjunction with non-overlapping region (INTERSECT).
        # Result should be NULL/empty table
        $maskfile = "$testin/pixmask_20x25_diastripe.fits";
        $regfile = "$testin/circle_no_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)&&sky=region($regfile)]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 117 )
    {
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates in conjunction with overlapping Shape expression (UNION).
        $maskfile = "$testin/pixmask_20x25_ON.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)||sky=Rect(4115,4100,4145,4160)+circle(4140,4100,43)]\" outfile=$outfile verbose=0";
    }

    if ( $testid == 118 )
    {
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates in conjunction with overlapping Shape expression (INTERSECT).
        $maskfile = "$testin/pixmask_20x25_ON.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)&&sky=Rect(4115,4100,4145,4160)+circle(4140,4100,43)]\" outfile=$outfile verbose=0";
    }


    # ------------------
    # MASK Rebinning
    # ------------------
    if ( $testid == 120 )  # fixed for bug14019, in 2 commands
    {
        $infile   = "$testin/indt046m121.fits";
        $maskfile = "$testin/pixmask_broad_thresh.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=${outfile}_tmp verbose=0;";
 	$test_string .= "$DMCOPY infile=\"${outfile}_tmp[bin sky=2]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 121 )  # fixed for bug14019, in 1 command
    {
        $maskfile = "$testin/pixmask_broad_thresh.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)][bin sky=2]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 122 )  # binning with mask bounds, having partial binning, runtime mask only
    {
        $maskfile = "$testin/pixmask_bthresh_sml.img";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)][bin sky=2]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 123 )  # binning range matching  mask bounds, having partial binning, runtime mask only
    {
        $maskfile = "$testin/pixmask_bthresh_sml.img";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)][bin x=3449.5:3650.5:2,y=3649.5:3950.5:2]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 124 ) # binning range out of mask bounds, no partial binning, runtime mask only
    {
        $maskfile = "$testin/pixmask_bthresh_sml.img";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)][bin sky=3097.0:4595.0:2]\" outfile=$outfile verbose=0";
    }

    if ( $testid == 125 ) # binning range within mask bounds, partial binning, runtime mask only
    {
        $maskfile = "$testin/pixmask_bthresh_sml.img";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)][bin sky=3650.0:3950.0:2]\" outfile=$outfile verbose=0";
    }
   if ( $testid == 126 ) # binning but runtime mask does not need the bin 
    {
        $maskfile = "$testin/pixmask_broad_thresh_bin2.img";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)][bin sky=2]\" outfile=$outfile verbose=0";
    }
   if ( $testid == 127 ) # binning 4 but runtime mask is binned in 2 
    {
        $maskfile = "$testin/pixmask_broad_thresh_bin2.img";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)][bin sky=4]\" outfile=$outfile verbose=0";
    }
   if ( $testid == 128 ) # binning not multiple, catch the error
    {
        $maskfile = "$testin/pixmask_broad_thresh_bin2.img";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)][bin sky=3]\" outfile=$outfile verbose=0";
    }

   if ( $testid == 129 ) # checking match coords and catch error of the mis-matching Offset
    {
        $maskfile = "$testin/pixmask_broad_thresh_filt.img";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskfile)]\" outfile=$outfile verbose=0";
    }
   if ( $testid == 130 ) #  test for fixing of bug#14051
    {
 	$outfile = "$testout/${basename}.txt";
	$savfile = "$testsav/${basename}.txt";

        $maskfile = "$testin/pixmask_4425_b1.img";
	$test_string  = "$DMLIST infile=\"${infile}[sky=mask($maskfile)]\" opt=counts outfile=$outfile verbose=0 ;";
	$test_string .= "$DMSTAT infile=\"${infile}[sky=mask($maskfile)][bin sky=1]\" sig- cen- | grep sum  >> $outfile ;";
 	$test_string .= "$DMSTAT infile=\"${infile}[sky=mask($maskfile)][bin sky=2]\" sig- cen- | grep sum  >> $outfile ;";
 	$test_string .= "echo \" \"";
    }
    if ( $testid == 131 ) #   from Table-event to create another table event with 2 masks
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 91;
	$infile = "$testsav/${basename}.$in_ext";
	$maskOne = "$testin/sky.mask";
	$maskTwo = "$testin/expno_vs_pi.mask";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne), (expno,pi)=mask($maskTwo)]\" outfile=${outfile} verbose=0 ";
    }
    if ( $testid == 132 ) #  test for fixing of bug#14270
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 131;
	$infile = "$testsav/${basename}.$in_ext";
        $test_string = "$DMCOPY infile=\"${infile}[bin sky=1]\" outfile=${outfile} verbose=0 ";
    }
    if ( $testid == 133) #  testing passing coords checking  in Table-evt input
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 91;
	$infile = "$testsav/${basename}.$in_ext";
	$maskOne = "$testin/sky.mask";
	$maskTwo = "$testin/expno_vs_pi.mask";
        $test_string  = "$DMCOPY infile=\"${maskOne}[bin sky=2]\" outfile=${outfile}_sky_s2.mask verbose=0 ; ";
        $test_string .= "$DMCOPY infile=\"${infile}[sky=mask(${outfile}_sky_s2.mask), (expno,pi)=mask($maskTwo)]\" outfile=${outfile} verbose=0 ";
    }
    if ( $testid == 134 ) #  test (expno,pi) subspace is remained, running image-event with two existing masks, filtered with a mask
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 132;
	$infile = "$testsav/${basename}.$in_ext";
	$maskOne = "$testin/sky.mask";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)]\" outfile=${outfile} verbose=0 ";
    }
     if ( $testid == 135 ) #  testing the binspec descriptor being catched and processed
     {
	$basename = sprintf "dt%.3dm%.3d", $id, 131;
	$infile = "$testsav/${basename}.$in_ext";
	$maskOne = "$testin/sky.mask";
	$maskTwo = "$testin/expno_vs_pi.mask";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne), (expno,pi)=mask($maskTwo)][bin expno=2,pi=2]\" outfile=${outfile} verbose=0 ";
     }

    if ( $testid == 136 ) # expected fail: binspec is not matching exisitng mask's, bin sky=1
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 91;
	$infile = "$testsav/${basename}.$in_ext";
	$maskOne = "$testin/sky_s2.mask";
	$maskTwo = "$testin/expno_vs_pi.mask";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne), (expno,pi)=mask($maskTwo)][bin sky=1]\" outfile=${outfile} verbose=0 ";
    }

    if ( $testid == 137 ) # expected fail: binspec is not matching exisitng mask's, bin sky=3
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 91;
	$infile = "$testsav/${basename}.$in_ext";
	$maskOne = "$testin/sky_s2.mask";
	$maskTwo = "$testin/expno_vs_pi.mask";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne), (expno,pi)=mask($maskTwo)][bin sky=3]\" outfile=${outfile} verbose=0";
    }
    if ( $testid == 138 ) #  expected fail: image-events with (expno,pi) subspace but no block column descriptor
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 132;
	$infile = "$testsav/${basename}.$in_ext";
	$maskOne = "$testin/sky.mask";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne), (expno,pi)=mask($maskTwo)]\" outfile=${outfile} verbose=0";
    }
    if ( $testid == 139 )  # cover SL-21
    {
	$infile  = "$testin/SL21_myimg";
	$maskOne = "$testin/SL21_2_step.img";
        $test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)][bin sky=2]\" outfile=${outfile} verbose=0";
    }
    # ------------------
    # MASK Consolidation
    # ------------------

    if ( $testid == 150 ) #c01
    {
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates followed by overlapping mask (40x30).
        $maskOne = "$testin/pixmask_20x25_diastripe.fits";
        $maskTwo = "$testin/pixmask_40x30_overlap.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)]\" outfile=${outfile}_tmp verbose=0; $DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";
    }
    if ( $testid == 151 ) #c01_2
    {
	# Postage stamp pixel mask (20x25) on sky(x,y) physical coordinates in conjunction with overlapping mask (40x30).
        #  EXPECTED FAILURE: syntax error to have multiple masks in same filter expression.
        $maskOne = "$testin/pixmask_20x25_diastripe.fits";
        $maskTwo = "$testin/pixmask_40x30_overlap.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)&&sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";
    }
    if ( $testid == 152 ) #c02
    {
	# Postage stamp pixel mask (40x30) on sky(x,y) physical coordinates followed by overlapping mask (20x25)
        # Same as #150, opposite order, should be same result..
        $maskTwo = "$testin/pixmask_20x25_diastripe.fits";
        $maskOne = "$testin/pixmask_40x30_overlap.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)]\" outfile=${outfile}_tmp verbose=0; ";
	$test_string .= "$DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";
    }
    if ( $testid == 153 ) #c02_2
    {
	# Postage stamp pixel mask (40x30) on sky(x,y) physical coordinates in conjunction with overlapping mask (20x25).
        #  EXPECTED FAILURE: syntax error to have multiple masks in same filter expression.
	#    to be implemented: error out due to the rule of "one Mask per command line"
        $maskTwo = "$testin/pixmask_20x25_diastripe.fits";
        $maskOne = "$testin/pixmask_40x30_overlap.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)&&sky=mask($maskTwo)]\" outfile=${outfile} verbose=0"
    }
    if ( $testid == 154 ) #c03
    {
	# Postage stamp pixel mask (20x25) in conjunction (UNION) with region, followed by overlapping mask (40x30).
        $maskOne = "$testin/pixmask_20x25_diastripe.fits";
        $maskTwo = "$testin/pixmask_40x30_overlap.fits";
        $regfile = "$testin/circle_no_overlap.reg";

	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)||sky=region($regfile)]\" outfile=${outfile}_tmp verbose=0; ";
	$test_string .= "$DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";

    }
    if ( $testid == 155 ) #c03_2
    {
	# Postage stamp pixel mask (20x25) in conjunction (UNION) with region AND intersected overlapping mask (40x30).
        #  EXPECTED FAILURE: syntax error to have multiple masks in same filter expression.
        $maskOne = "$testin/pixmask_20x25_diastripe.fits";
        $maskTwo = "$testin/pixmask_40x30_overlap.fits";
        $regfile = "$testin/circle_no_overlap.reg";
	$test_string = "$DMCOPY infile=\"${infile}[(sky=mask($maskOne)||sky=region($regfile))&&sky=mask($maskTwo)]\" outfile=${outfile} verbose=0"

    }
    if ( $testid == 156 ) #c04
    {
	# IMAGE - Postage stamp pixel mask (20x25), followed by overlapping mask (40x30).
        $maskOne = "$testin/pixmask_20x25_diastripe.fits";
        $maskTwo = "$testin/pixmask_40x30_overlap.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)]\" outfile=${outfile}_tmp   verbose=0; ";
	$test_string .= "$DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";

    }
    if ( $testid == 157 ) #c04_2
    {
	# IMAGE - Postage stamp pixel mask (20x25), in conjuction with (INTERSECT) overlapping mask (40x30).
        #  EXPECTED FAILURE: syntax error to have multiple masks in same filter expression.
        $maskOne = "$testin/pixmask_20x25_diastripe.fits";
        $maskTwo = "$testin/pixmask_40x30_overlap.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)&&sky=mask($maskTwo)]\" outfile=${outfile} verbose=0"
        
    }
    if ( $testid == 158 ) #c05 == c03?
    {
	# Postage stamp pixel mask (20x25) in conjunction (UNION) with region, followed by overlapping mask (40x30).
        $maskOne = "$testin/pixmask_20x25_diastripe.fits";
        $maskTwo = "$testin/pixmask_40x30_overlap.fits";
        $regfile = "$testin/circle_no_overlap.reg";

	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)||sky=region($regfile)]\" outfile=${outfile}_tmp verbose=0; ";
	$test_string .= "$DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";

    }
    if ( $testid == 159 ) #c06
    {
	# Postage stamp pixel mask (40x30) in conjunction (UNION) with region, followed by overlapping mask (20x25).
        # Modified sequence of #158
        $maskTwo = "$testin/pixmask_20x25_diastripe.fits";
        $maskOne = "$testin/pixmask_40x30_overlap.fits";
        $regfile = "$testin/circle_no_overlap.reg";

	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)||sky=region($regfile)]\" outfile=${outfile}_tmp verbose=0; ";
	$test_string .= "$DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";

    }
    if ( $testid == 160 ) #c07
    {
	# IMAGE - Postage stamp pixel mask (40x30), in conjuction with (INTERSECT) overlapping mask (20x25).
        #   Modified sequence from #156
        $maskTwo = "$testin/pixmask_20x25_diastripe.fits";
        $maskOne = "$testin/pixmask_40x30_overlap.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)]\" outfile=${outfile}_tmp verbose=0; $DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";

    }
    if ( $testid == 161 ) #c08
    {
	# Postage stamp pixel mask (40x30) in conjunction (INTERSECT) with region, followed by overlapping mask (20x25).
        $maskOne = "$testin/pixmask_40x30_overlap.fits";
        $maskTwo = "$testin/pixmask_20x25_diastripe.fits";
        $regfile = "$testin/circle_overlap.reg";

	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)&&sky=region($regfile)]\" outfile=${outfile}_tmp verbose=0; $DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";

    }
    if ( $testid == 162 ) #c09
    {
	# IMAGE - Postage stamp pixel mask (20x25) in conjunction (INTERSECT) with region, followed by overlapping mask (20x25).
        $maskOne = "$testin/pixmask_20x25_diastripe.fits";
        $maskTwo = "$testin/pixmask_40x30_overlap.fits";
        $regfile = "$testin/circle_overlap.reg";

	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)&&sky=region($regfile)]\" outfile=${outfile}_tmp verbose=0; $DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";

    }
    if ( $testid == 163 ) #c10
    {
	# Complex expression: ((Mask && Region1)||Region2) ,followed by overlapping mask (20x25).
        $maskOne = "$testin/pixmask_40x30_overlap.fits";
        $maskTwo = "$testin/pixmask_20x25_diastripe.fits";
        $regOne = "$testin/circle_overlap.reg";
        $regTwo = "$testin/circle_no_overlap.reg";

	$test_string = "$DMCOPY infile=\"${infile}[(sky=mask($maskOne)&&sky=region($regOne))||sky=region($regTwo)]\" outfile=${outfile}_tmp verbose=0; $DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";

    }
    if ( $testid == 164 ) #c11
    {
	# Complex expression: (Mask||Region1),followed by (Mask2&&Region2).
        $maskOne = "$testin/pixmask_40x30_overlap.fits";
        $maskTwo = "$testin/pixmask_20x25_diastripe.fits";
        $regOne = "$testin/circle_no_overlap.reg";
        $regTwo = "$testin/circle_overlap.reg";

	$test_string = "$DMCOPY infile=\"${infile}[(sky=mask($maskOne)||sky=region($regOne))]\" outfile=${outfile}_tmp verbose=0; $DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)&&sky=region($regTwo)]\" outfile=${outfile} verbose=0";

    }

    if ( $testid == 165 ) #c12
    {
	# Complex expression: ((Mask&&Region1)||Region2), followed by (Mask2).
        $maskOne = "$testin/pixmask_40x30_overlap.fits";
        $maskTwo = "$testin/pixmask_20x25_diastripe.fits";
        $regOne = "$testin/circle_overlap.reg";
        $regTwo = "$testin/circle_no_overlap.reg";

	$test_string = "$DMCOPY infile=\"${infile}[(sky=mask($maskOne)&&sky=region($regOne))||sky=region($regTwo)]\" outfile=${outfile}_tmp verbose=0; $DMCOPY infile=\"${outfile}_tmp[sky=mask($maskTwo)]\" outfile=${outfile} verbose=0";

    }

    # ----------------------------------------------------
    # multi-masks are not unioned as GTIs are encountered
    # ----------------------------------------------------
  
    if ( $testid == 202 )  # 202,205,206,207,208 are all due to the fixings of SL19 and SL20
    {
        # Merge 2 files which have masks not overlapped
        $infile1 = "$testin/indt046m${testid}_1.fits";
        $infile2 = "$testin/indt046m${testid}_2.fits";
	$test_string = "$DMMERGE \"${infile1},${infile2}\" outfile=$outfile outBlock='MERGED'";
    }
    if ( $testid == 205 )  
    {
        # Merge 5 files which  masks not overlapped
        $infile1 = "$testin/indt046m202_1.fits";
        $infile2 = "$testin/indt046m202_2.fits";
        $infile3 = "$testin/indt046m${testid}_1.fits";
        $infile4 = "$testin/indt046m${testid}_2.fits";
        $infile5 = "$testin/indt046m${testid}_3.fits";
	$test_string = "$DMMERGE \"${infile1},${infile2},${infile3},${infile4},${infile5}\" outfile=$outfile outBlock='MERGED'";
    }

    if ( $testid == 206 )
    {
        # Merge 2 files which have mask&circle, two circles are overlapped
        $infile1 = "$testin/indt046m${testid}_1.fits";
        $infile2 = "$testin/indt046m${testid}_2.fits";
	$test_string = "$DMMERGE \"${infile1},${infile2}\" outfile=$outfile outBlock='MERGED'";
    }

    if ( $testid == 207 )
    {
        # Merge 4 files which have mask&shape, two shapes are overlapped
        $infile1 = "$testin/indt046m206_1.fits";
        $infile2 = "$testin/indt046m206_2.fits";
        $infile3 = "$testin/indt046m${testid}_1.fits";
        $infile4 = "$testin/indt046m${testid}_2.fits";
	$test_string = "$DMMERGE \"${infile1},${infile2},${infile3},${infile4}\" outfile=$outfile outBlock='MERGED'";
    }
 
    if ( $testid == 208 )
    {
        # Merge 2 files which have masks overlapped
        $infile1 = "$testin/indt046m${testid}_1.fits";
        $infile2 = "$testin/indt046m${testid}_2.fits";
	$test_string = "$DMMERGE \"${infile1},${infile2}\" outfile=$outfile outBlock='MERGED'";
    }

    # ---------------------------------------
    # multi-masks with or-Op components, etc.
    # ---------------------------------------
    if ( $testid == 209 )
    {
        # Merge 2 files which have masks overlapped
        $infile1 = "$testin/SL9_a1.fits";
        $infile2 = "$testin/SL9_xa1.fits";
	$test_string = "$DMMERGE \"${infile1},${infile2}\" outfile=$outfile";
    }
    if ( $testid == 210 )
    {
        # Merge 2 files which have masks overlapped
        $infile1 = "$testin/SL9_xa1.fits";
        $infile2 = "$testin/SL9_a1.fits";
	$test_string = "$DMMERGE \"${infile1},${infile2}\" outfile=$outfile";
    }
    if ( $testid == 211 )
    {
        # Merge 2 files which have masks overlapped
        $infile1 = "$testin/SL9_evt_circle_m.fits";
        $infile2 = "$testin/SL9_evt_circle_m_exc.fits";
	$test_string = "$DMMERGE \"${infile1},${infile2}\" outfile=$outfile";
    }
    # --------------------------------------------
    # multi-masks are unioned as there are no GTIs
    # ---------------------------------------------
    if ( $testid == 212 )
    {
        # Merge 4 files which have masks overlapped
        $infile1 = "$testin/SL20_0.raw.crop.msk";
	$infile2 = "$testin/SL20_1.raw.crop.msk";
	$infile3 = "$testin/SL20_2.raw.crop.msk"; # output file size is too big to be ignored here
	$infile4 = "$testin/SL20_3.raw.crop.msk"; # output file size is too big to be ignored here
	$test_string = "$DMMERGE \"${infile1},${infile2}\" outfile=$outfile";
    }

    # ------------------------------------------------------------------------------------------
    # combine 5 polygons (4 overlapped and 1 not overlapped), output 2 SS components 
    # ------------------------------------------------------------------------------------------
    if ( $testid == 213 )
    {
        # Merge 5 files which have polys overlapped
        $infile1 = "$testin/SL80_poly01237.evt";
	$test_string = "$DMCOPY \"${infile1}[subspace -time,-expno,-ccd_id]\" outfile=$outfile";
    }
    # ------------------------------------------------------------------------------------------
    # combine 5 masks (4 overlapped and 1 not overlapped), output 2 SS components 
    # ------------------------------------------------------------------------------------------
    if ( $testid == 214 )
    {
        # Merge 5 files which have 4 masks overlapped
        $infile1 = "$testin/SL80_mask01237.evt";
	$test_string = "$DMCOPY \"${infile1}[subspace -time,-expno,-ccd_id]\" outfile=$outfile";
    }
    # ------------------------------------------------------------------------------------------------------------
    # combine 5 polygons (4 overlapped and 1 not overlapped), output 1 SS component, the component order matters 
    # ------------------------------------------------------------------------------------------------------------
    if ( $testid == 215 )
    {
        # Merge 5 files which have polys overlapped
        $infile1 = "$testin/SL80_poly07123.evt";
	$test_string = "$DMCOPY \"${infile1}[subspace -time,-expno,-ccd_id]\" outfile=$outfile";
    }
    # ------------------------------------------------------------------------------------------------------------
    # combine 5 polygons (4 overlapped and 1 not overlapped), output 1 SS component, the component order matters 
    # ------------------------------------------------------------------------------------------------------------
    if ( $testid == 216 )
    {
        # Merge 5 files which have masks overlapped
        $infile1 = "$testin/SL80_mask07123.evt";
	$test_string = "$DMCOPY \"${infile1}[subspace -time,-expno,-ccd_id]\" outfile=$outfile";
    }


    # ------------------
    # Other:
    # ------------------
    if ( $testid == 201 )
    {
	# Straight copy of file with MASK subspace.. test transfer
	$test_string = "$DMCOPY infile=${infile} outfile=$outfile verbose=0";
    }

    if ( $testid == 203 )
    {
        # Check mask-file data close-to-zero, *_MIN, [1,2] blocks are both image
        $maskOne     = "$testin/pixmask_minval_00000000000000000ones.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)]\"  outfile=$outfile"
    }

    if ( $testid == 204 )
    {
        # Check mask-file user-defined NULL, looking for 'BLANK' keyword,  [1,2] blocks are both image
        $maskOne = "$testin/pixmask_null_nulls2.fits";
	$test_string = "$DMCOPY infile=\"${infile}[sky=mask($maskOne)]\"    outfile=$outfile"
    }
    
    # ------------------------------------------------------------------------------------------------------------
    # single mask subspace expression 
    # ------------------------------------------------------------------------------------------------------------
    if ( $testid == 221 )
    {
        #  input: one mask only subspace expression in 'mask' and the mask block is missing
	# output: clean off the literal 'mask' and so clean off DSTYP/DSVAL keyword 
	$test_string = "$DMCOPY infile=${infile}  outfile=$outfile verbose=0";
    }

     
    # ------------------------------------------------------------------------------------------------------------
    # mask&Shapes subspace expression 
    # ------------------------------------------------------------------------------------------------------------
    if ( $testid == 222 )
    {
        #  input: mask&shapes subspace expression and the mask block is missing
	# output: drop off the literal 'mask'  and reatin the shapes 
	$test_string = "$DMCOPY infile=${infile}  outfile=$outfile verbose=0";
    }

    # ------------------------------------------------------------------------------------------------------------
    # mask&shapes subspace expression through multi ss cpts, output warning once for multi warn instances. 
    # ------------------------------------------------------------------------------------------------------------
    if ( $testid == 223 )
    {
        #  input: mask&shapes|mask2&shapes subspace expression with ss cpts>1 and the mask blocks is missing
	# output: clean off the literal 'mask' and so clean off DSTYP/DSVAL keyword
	#         print warning just once instead of multi-times because of multi ss cpts
	$test_string = "$DMCOPY infile=${infile}  outfile=$outfile verbose=2";
    }
    $test_string .= " >> $logfname 2>&1";

    # --------------------------------------------------------------------------------
    # run the test
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);

    if ( $block_types{"fail"} )
    { # Expected Failure test check.
        if ( $rc == 0 )
	{
	    $err_msg = "  ERROR: Expected failure did not occur.\n";
	} 
	$rc = 0; 
    }

    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 )
    {
      $err_msg = "ERROR: $toolname failed to run\n";
    }
   
    # --------------------------------------------------------------------------------
    # validate results
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff ); 
     }
    }
    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      if ( $testid != 85 ) {
     
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
     } else {
      foreach my $block ( @blocks )
      {
        $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff ); 
      }
     }

    }
    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt_list", $savfile, "$testout/${basename}_savfilt_list", \@tolkeys, $logfname, $test_diff, "
" );
    }
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_filt_text", $savfile, "$testout/${basename}_savfilt_text", \%test_words, $logfname, $test_diff )
;
    }

    # --------------------------------------------------------------------------------
    # report results
    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        #push (@faillist, $kernelName);
        #$errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );
    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;
    
    # --------------------------------------------------------------------------------
    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ( $skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
    }

  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
