#! /bin/sh

# 08 January 2013

# This is the official template for pipetool regression test scripts.
# In addition to supporting the "SHORTTEST" option, this script also
# allows the user to run individual subtests from the command line.
# The script will accept a series of test identifiers, generally of
# the form "test1" "test2" ... which are to be run.

# Portions of the script which must be customized are marked with "!!",
# below.

# The complete list of tests must be placed in "alltests", !!3, below.
# The test[s] for the SHORTTEST must be placed in "shortlist", !!4 below.


# !!1
# mytool.t
# test script for mytool


# !!2
# syntax:
# mytool.t [<testid> ... ]
 
# Developer's comments:
#
# default CALDB env for the regression test
# setenv CALDB /data/regression_test/development/CALDB4
#
# ---------------------------------------------------------



######################################################################
# subroutine
# error_exit <message>
# Fatal error exit

error_exit()
{
  echo "$1" | tee -a $LOGFILE
  echo "${toolname} : FAIL" | tee -a $LOGFILE
  exit 1
}

######################################################################
# subroutine
# filter infile outfile
# filters out CHECKSUM, Dataset, CREATOR,da/analysis/caltools/calquiz HISTORY, DATASUM, 
#             ASCDSVER, HISTNUM, and DATE
# To filter additional keywords, add s/KEYWORD/Dataset/g; for each.

filepath_filter()
{

# change the pathed files to descripitives (Linux version)
# from
#  PIXLIB Version: 4.0.0             -> PIXLIB Version: <vers#>
#  /home/hhe/cxcds_param4/geom.par:  -> <param-path>/geom.par:
#
#  instruments  = /data/regression_test/development/CALDB4/data/chandra/default/geom/telD1999-07-23geomN0005.fits(CALDB)                              ->
# instruments
# = <CALDB-path>/data/chandra/default/geom/telD1999-07-23geomN0005.fits(CALDB)
# ...
#
 
 if test $# -eq 3; then
     sed -e '/PIXLIB Version:/d' $1 | sed -e '/ = \//d' | sed -e '/\//d'>$2.tmp&&mv $2.tmp $2
 elif test "x`uname`" = "xSunOS"; then
     sed -e '/PIXLIB Version:/d' $1 | sed -e '/ = \//d' | sed -e '/^\//d'>$2.tmp&&mv $2.tmp $2
 else
     sed -e '/PIXLIB Version:/ c PIXLIB Version: <vers#>' $1 | sed -e '/\/geom.par:/ c <param-path>\/geom.par:' | gawk -F"/data/chandra" '{for (i=1;i<=NF;i++){{print $i}}}' | sed -e 's/\/default/<CALDB-path>\/data\/chandra\/default/g' | gawk -F" = /" '{{print $1}}' > $2
 fi

 if test -s $2 ; then
      :
 else
     echo "ERROR: file $1 is of zero length" >> $LOGFILE
   #  Indicate failure, but do not exit.
     mismatch=0
 fi

}

######################################################################
# subroutine
# find_tool <toolname>
# checks that tool exists and is runnable
find_tool()
{
  s1=`type $1`
  s2=`echo $s1 | awk -F" " '{ print $3}'`
  if test -x $s2 ; then
      :
  else
    error_exit "tool $1 not found"
  fi

}
######################################################################
# subroutine
# exportenv <ket> <val>
# portably export environ-variable
exportenv()
{
 key=`echo $1`
 val=`echo $2`
 if test "x`uname`" = "xSunOS"; then
     if test $key = "CALDB"; then 
	 CALDB=$val; export CALDB 
     elif test $key = "CALDBCONFIG";then 
	 CALDBCONFIG=$val; export CALDBCONFIG
     elif test $key = "CALDBALIAS";then
	 CALDBCONFIG=$val; export CALDBALIAS
     elif test $key = "PATH";then
	 PATH=$val;export PATH
     fi
 else
     export $key=$val
 fi

 if test $key = "CALDB"; then 
     echo "export $key=" $CALDB; 
 elif test $key = "CALDBCONFIG";then 
     echo "export $key=" $CALDBCONFIG
 elif test $key = "CALDBALIAS";then
     echo "export $key=" $CALDBALIAS
 elif test $key = "PATH";then
     echo "export $key=" $PATH
 fi

}

######################################################################
# subroutine
# texec_dir <message>
# Fatal error exit

set_texec_dir()
{

    curpath=`dirname $0`
    thetool=$1
    if test -d "${curpath}/${thetool}"; then	
	TEXEC_DIR="${curpath}/${thetool}";
    elif test -d "$ASCDS_INSTALL/test/bin/${thetool}"; then
	TEXEC_DIR="$ASCDS_INSTALL/test/bin/${thetool}";
    else
	echo "ERROR: Can't find ${thetool} test scripts.";
	exit 1;
    fi
    exportenv PATH $TEXEC_DIR:$PATH
}

######################################################################
# Initialization

# !!3
toolname="pixlib"

# set up list of tests
# !!4
#alltests="lookup_time"
alltests="pixdefault coords 4by4matrix config errsta mk_aspect grating \
          pixutils pixucase mult_init map_aimpnt prtconst\
          mirgrt_3 mirgrt_4 mirgrt_5 mirgrt_6 mirgrt_7 \
          align_0 align_1 align_2 align_3 \
          pixplane pixtdet pixtap moving_sim moving_sim_xrcf \
          fpsys shell_prop chip_prop ext_chip ext_chips \
          corner_tdet corner_tdet_xrcf corner_fpc corner_lsi \
          c2s_acisi0 c2s_hrci c2s_hrcs1 c2s_hrcs2 c2s_hsi c2s_acisc1\
          hist"

if test "x`uname`" != "xDarwin"; then
    alltests="${alltests} grating_1kev"
fi
 

# "short" test to run
# !!5
shortlist="ext_chips ext_chip align_0 align_1 align_2 align_3"

set_texec_dir $toolname

# compute date string for log file
DT=`date +'%d%b%Y_%T'`


# convenience definitions
OUTDIR=$TESTOUT/$toolname
SAVDIR=$TESTSAV/$toolname
INDIR=$TESTIN/$toolname
LOGDIR=$TESTLOG/$toolname

# set up log file name
LOGFILE=$LOGDIR/${toolname}_log.$DT

#get rid of old logs
rm -f $LOGDIR/${toolname}_log.*



# Any tests specified on command line?
if test $# -gt 0; then
  # yes, do those tests
  testlist=$*
else
  # No, see if we are to do "short" test
  if test "x$SHORTTEST" = "x" ; then
    # No, do everything
    testlist=$alltests
  else
    # yes, do short test
    testlist=$shortlist
  fi
fi

# Make sure we have a log directory
if test -d $LOGDIR ; then
 :
else
  mkdir -p $LOGDIR 
  if test $? -ne 0 ; then
    error_exit ""
  fi
fi


# Make sure we have an output directory
if test -d $OUTDIR ; then
 :
else
  mkdir -p $OUTDIR >> $LOGFILE 2>&1
  if test $? -ne 0 ; then
    error_exit "can't create output directory $OUTDIR"
  fi
fi

# check for directory environment variables
if test "x${TESTIN}" = "x" -o "x${TESTOUT}" = "x" -o "x${TESTSAV}" = "x" \
   -o "x${TESTLOG}" = "x" ; then
  error_exit "one or more of TESTIN/TESTOUT/TESTSAV/TESTLOG not defined" 
fi


# check for tools
# if a utility is used in the form "utility <args> > outfile", and 'utility'
# cannot be run, 'outfile' will still be created.  If utility is used on 
# both the output and reference files of a tool the resultant utility output 
# files will both exist and be empty, and will pass a diff.

#find_tool calquiz



# announce ourselves
echo ""
echo "${toolname} regression tests" | tee $LOGFILE
echo ""

# All parameters except verbose should be set anyway, but clear them
# to be safe.
punlearn geom.par

script_succeeded=0

######################################################################
# Begin per-test loop

for testid in $testlist
do
    
  # delete old outputs
  rm -f $OUTDIR/${testid}*

  # Set up file names
  outfile=$OUTDIR/${testid}.txt
  savfile=$SAVDIR/${testid}.txt

  echo "" >> $LOGFILE
  echo "running $testid" >> $LOGFILE

  ####################################################################
  # run the tool
  case ${testid} in

      pixdefault) command_str=" ${testid} $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      coords) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      
      4by4matrix) command_str=" ${testid} flight > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      grating_1kev) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      config) command_str="${testid} $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      errsta) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      hist) command_str=" ${testid} $outfile" 
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

     prtconst) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      pixutils) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      pixucase) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      mult_init) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      map_aimpnt) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      mk_aspect) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      grating) command_str=" ${testid} flight > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;


      mirgrt_3)  command_str="echo 3 | mk_mirgrt flight  > $outfile"
 	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      mirgrt_4) command_str="echo 4 | mk_mirgrt flight  > $outfile"
 	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      mirgrt_5) command_str="echo 5 | mk_mirgrt flight  > $outfile"
 	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;     
      mirgrt_6) command_str="echo 6 | mk_mirgrt flight  > $outfile"
 	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      mirgrt_7) command_str="echo 7 | mk_mirgrt flight  > $outfile"
 	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      align_0) command_str="echo 0 | mk_align xrcf ${outfile}_tmp 1 > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  command_str="${command_str}; cat ${outfile}_tmp >> $outfile;rm ${outfile}_tmp" 
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      align_1) command_str="echo 1 | mk_align xrcf ${outfile}_tmp 1 > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  command_str="${command_str}; cat ${outfile}_tmp >> $outfile;rm ${outfile}_tmp" 
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      align_2) command_str="echo 2 | mk_align xrcf ${outfile}_tmp 1 > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  command_str="${command_str}; cat ${outfile}_tmp >> $outfile;rm ${outfile}_tmp" 
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      align_3) command_str="echo 3 | mk_align xrcf ${outfile}_tmp 1 > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  command_str="${command_str}; cat ${outfile}_tmp >> $outfile;rm ${outfile}_tmp" 
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      pixplane) command_str=" ${testid} flight > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      pixtdet) command_str=" ${testid} layout > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      pixtap) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      moving_sim) command_str="${testid} flight > $outfile"
 	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      moving_sim_xrcf) command_str="moving_sim xrcf > $outfile"
 	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      fpsys) command_str="${testid} xrcf > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      
      corner_tdet) command_str="corners flight tdet > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      corner_tdet_xrcf) command_str="corners xrcf tdet > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      corner_fpc) command_str="corners flight fpc > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      corner_lsi) command_str="corners flight lsi > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      c2s_acisi0) command_str="cpc2stf -ss acis-i0 > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      c2s_hrci) command_str="cpc2stf -ss hrc-i > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      c2s_hrcs1) command_str="cpc2stf -ss hrc-s1 > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      c2s_hrcs2) command_str="cpc2stf -ss hrc-s2 > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      c2s_hsi) command_str="cpc2stf -ss hsi > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      c2s_acisc1) command_str="cpc2stf -ss acis-c1 > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      dataconv_1) command_str="print 1 3.2 1 | dataconv > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      dataconv_2) command_str="print 1 3.2 0 | dataconv > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      dataconv_3) command_str="print 1 5.6 1 | dataconv > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;
      dataconv_4) command_str="print 1 3.6 0 | dataconv > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      shell_prop) command_str="${testid}  > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

      chip_prop) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

     ext_chip) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

     ext_chips) command_str=" ${testid} > $outfile"
	  echo $command_str  | tee -a $LOGFILE
	  eval $command_str  | tee -a $LOGFILE  2>&1
	  ;;

  esac


#--- get file without path from <tool>.par's 'outfile' field -----
#line1 read .par and single out 'outfile=' line
#line2 remove string "outfile = "
#line3 remove string "File(s) found"
#line4 remove the file path
 # plist ${toolname} | grep outfile |\
 # sed 's/'outfile\ \=\ '//g'       |\
 # sed 's/'File\(s\)\ found'//g'    |\
 # sed 's/\/*[a-Z,0-9, _]*\///g'     > $outfile

  mismatch=1
  if  test -f $outfile; then
      x=1
  else
      echo "not found " $outfile  >> $LOGFILE
      mismatch=0
  fi
  if  test -f $savfile; then
      x=1
  else
      echo "not found " $savfile  >> $LOGFILE
      mismatch=0
  fi
        
  ####################################################################
  # check the outputs

  # if different tests need different kinds of comparisons, use a 
  #  case ${testid} in...  here

  # working the sav-data in the tmp-dir
  cat $savfile >  ${outfile}_s

  filepath_filter ${outfile}   ${outfile}_o2 "rm_path"
  filepath_filter ${outfile}_s ${outfile}_s2 "rm_path"
  
  savfile=${outfile}_s2
  outfile=${outfile}_o2

  if test $mismatch -ne 0; then
      diff $outfile $savfile  > /dev/null 2>>$LOGFILE
      if  test $? -ne 0 ; then
	  echo "ERROR: MISMATCH in $outfile" >> $LOGFILE
	  mismatch=0
      fi	  
  fi
  
  ####################################################################
  # FITS image  (duplicate for as many images per test as needed)

  # check image
  # !!13
  # dmimgcalc "$outfile[1]" "$savfile[1]" none tst verbose=0   2>>$LOGFILE
  # if test $? -ne 0; then
  #   echo "ERROR: DATA MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  #  Check the header of the image

  # !!14
  # dmlist $outfile header > $OUTDIR/${testid}.dmp1  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1 $OUTDIR/${testid}.dmp2  2>>$LOGFILE

  # !!15
  # dmlist $savfile header > $OUTDIR/${testid}.dmp1_std  2>>$LOGFILE
  # keyfilter $OUTDIR/${testid}.dmp1_std $OUTDIR/${testid}.dmp2_std \
  #            2>>$LOGFILE

  # compare
  # !!16
  # diff $OUTDIR/${testid}.dmp2 $OUTDIR/${testid}.dmp2_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: HEADER MISMATCH in $outfile" >> $LOGFILE
  #   mismatch=0
  # fi

  ######################################################################
  # ascii files
  # !!17
  # diff $OUTDIR/$outfile $OUTDIR/$outfile_std > 
  #       /dev/null 2>>$LOGFILE
  # if  test $? -ne 0 ; then
  #   echo "ERROR: TEXT MISMATCH in $OUTDIR/${testid}.txt" >> $LOGFILE
  #   mismatch=0
  # fi

  ####################################################################
  # Did we get an error?
  if test $mismatch -eq 0 ; then
    # Yes
      echo "${testid} NOT-OK"
      script_succeeded=1
  else
    # No
      echo "${testid} OK"
  fi
  echo ""


done
# end per-test loop
######################################################################


######################################################################
# report results

# blank line
echo ""

if test $script_succeeded -eq 0; then
    echo "${toolname} : PASS" | tee -a $LOGFILE
else
    echo "${toolname} : FAIL" | tee -a $LOGFILE
fi

echo "log file in ${LOGFILE}"

exit $script_succeeded
